#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "conf.h"
#include "debug.h"
#include "gtkftpd.h"
#include "conf_defaultval.h"
#include "div.h"

#define CONF_FILE "/etc/gtkftpd"
#define LINE_LEN 256

char **file=NULL;
char *multiple_settings=NULL;

static void write_new_conf();
static char *default_value(const char *balise);

gboolean init_conf()
{
	char *line;
	FILE *fp=NULL;
	int nbline=0, empty_line=1, i=0;

	multiple_settings = (char *) malloc(1);
	memset(multiple_settings, 0, 1);
	if ( !(fp=fopen(CONF_FILE, "r+")) )
	{
		if ( !(fp=fopen(CONF_FILE, "w+")) )
		{
			perror("fichier de configuration");
			return(TRUE);
		}
	}
	while ( !feof(fp) )
	{
		if ( fgetc(fp)=='\n' )
		{
			if (!empty_line)
				nbline++;
			empty_line=1;
		} else 
				empty_line=0;
	}
	file = (char **) malloc(nbline*4 +4);
	memset(file, 0, nbline*4+4);
		
	line = (char *) malloc(LINE_LEN);
	fseek(fp, 0, SEEK_SET);
	while ( !feof(fp) )
	{
		memset(line, 0, LINE_LEN);
		fgets(line, LINE_LEN, fp);
		if ( feof(fp) )		break;
		if ( strlen(line)<2)	continue;
		if (line)
		{
			if ( line[strlen(line)-1]!='\n' )
			{
				fprintf(stderr, "Error in the config file: line > 256c\n");
				return(FALSE);
			}
			/*file[i] = (char *) malloc(strlen(line)+1);
			memset(file[i], 0, strlen(line)+1);
			strncpy(file[i], line, strlen(line));*/
			line[strlen(line)-1] = '\0';
			file[i] = strdup(line);
			i++;
		}
	}
	file[i] = NULL;
	free(line);
	fclose(fp);
	return(TRUE);
}

void desinit_conf()
{
	int i=-1;

	if (file)
	{
		write_new_conf();
		while (file[++i])
			free(file[i]);
		free(file);
	}
}


/* Si grep n'est pas null, supprime juste les settings le contenant */
void empty_conf(const char *grep)
{
	int i=-1, siz=0, j;
	char **newfile;

	if (file)
	{
		if (!grep)
		{
			while (file[++i])
				free(file[i]);
			free(file);
			file = malloc(4);
			memset(file, 0, 4);
		} else {
			while (file[++i])
				if ( !strstr(file[i], grep) )
					siz++;
			newfile = (char **) malloc(siz*4+4);
			memset(newfile, 0, siz*4+4);
			i=-1;
			j=0;
			while (file[++i])
			{
				if ( strstr(file[i], grep) && space_before_end(strstr(file[i],grep)) )
					free(file[i]);
				else
					newfile[j++] = file[i];
			}
			free(file);
			file = newfile;
		}
	}
}


char *get_setting(const char *balise)
{
	char *result=NULL, *setting;
	int i=-1;

	if (!balise)
	{
#ifdef DEBUG
		fprintf(stderr, "get_setting a ete appele avec une balise NULL => ignore");
#endif
		return(NULL);
	}
	while (file[++i])
	{
		if ( !strncasecmp(file[i], balise, strlen(balise)) && ( file[i][strlen(balise)]==' ' ||
					!file[i][strlen(balise)] || file[i][strlen(balise)]=='\n' || file[i][strlen(balise)]=='\t') )
		{
			setting = file[i]+strlen(balise)+1;
			result = strdup(setting);
			break;
		}
	}
	if (result==NULL)
		result = default_value(balise);
	return(result);
}


char *default_value(const char *balise)
{
	int i=-1;

	while (defaultval[++i].balise)
	{
		if ( !strncmp(defaultval[i].balise, balise, strlen(balise)) )
			return(defaultval[i].val);
	}
#ifdef DEBUG
		fprintf(stderr, "Error: can't find balise \"%s\" in the config file\n", balise);
#endif
	return(NULL);
}


char **get_settings(const char *balise)
{
	char separator = ' ';
	char *toparse, **result, *temp;
	int i, cptr=1;

	temp = toparse = get_setting(balise);
	if (!temp)
		return(NULL);
	for (i=0;i<strlen(toparse);i++)
	{
		if (*temp++==separator)
			cptr++;
	}
	result = (char **) malloc(cptr*4+4);
	memset(result, 0, cptr*4+4);
	for (i=0;i<cptr;i++)
	{
		temp = strrchr(toparse, separator);
		if (!temp)
			result[i] = strdup(toparse);
		else {
			result[i] = strdup(++temp);
			*--temp = 0;
		}
	}
	result[cptr] = NULL;
	return(result);
}


void put_setting_str(const char *balise, const char *value)
{
	int i=-1, newsetting=1;

	while (file[++i])
	{
		if ( !strncasecmp(file[i], balise, strlen(balise)) &&
				( (file[i][strlen(balise)]==' ') || file[i][strlen(balise)]=='\t' || file[i][strlen(balise)]=='\n' ) )
		{
			newsetting=0;
			file[i] = (char *) realloc(file[i], strlen(balise)+strlen(value)+2 );
			memset(file[i], 0, strlen(balise)+strlen(value)+2);
			strncpy(file[i], balise, strlen(balise));
			strncat(file[i], " ", 2);
			strncat(file[i], value, strlen(value)+1);
			break;
		}
	}

	if (newsetting)
	{
		file = (char **) realloc(file, i*4+8);
		file[i+1] = NULL;
		file[i] = (char *) malloc( strlen(balise)+strlen(value)+2 );
		memset(file[i], 0, strlen(balise)+strlen(value)+2);
		strncpy(file[i], balise, strlen(balise));
		strncat(file[i], " ", 1);
		strncat(file[i], value, strlen(value));
	}	
}


void put_setting_int(const char *balise, const int value)
{
	char val[11];		// nbre de chiffre d'un octet
	
	memset(val, 0, 11);
	sprintf(val, "%d", value);
	put_setting_str(balise, val);
}

void put_setting_str_and_deb_balise(const char *deb_balise, const char *end_balise, const char *val)
{
	char *balise;

	balise = (char *) malloc(strlen(deb_balise)+strlen(end_balise)+1);
	memset(balise, 0, strlen(deb_balise)+strlen(end_balise)+1);
	strncpy(balise, deb_balise, strlen(deb_balise));
	strncat(balise, end_balise, strlen(end_balise));
	put_setting_str(balise, val);
	free(balise);
}

void put_setting_str_and_deb_end_balise(const char *deb_balise, const char *mid_balise, const char *end_balise, const char *val)
{
	char *balise;

	balise = (char *) malloc(strlen(deb_balise)+strlen(mid_balise)+strlen(end_balise)+1);
	memset(balise, 0, strlen(deb_balise)+strlen(mid_balise)+strlen(end_balise)+1);
	strncpy(balise, deb_balise, strlen(deb_balise));
	strncat(balise, mid_balise, strlen(mid_balise));
	strncat(balise, end_balise, strlen(end_balise));
	put_setting_str(balise, val);
	free(balise);
}

void put_one_setting_str_of_multiple(const char *val)
{
	multiple_settings = (char *) realloc(multiple_settings, strlen(multiple_settings)+strlen(val)+2);
	if (!multiple_settings)
	{
		perror("Erreur d'allocation memoire");
		gtk_main_quit();
		exit(0);
	}
	if (strlen(multiple_settings)>1)
	{
		strcat(multiple_settings, " ");
		strcat(multiple_settings, val);
	} else
		strcat(multiple_settings, val);
}


void put_multiple_setting(const char *balise)
{
	put_setting_str(balise, multiple_settings);
	free(multiple_settings);
	multiple_settings = (char *) malloc(1);
	memset(multiple_settings, 0, 1);
}


static void write_new_conf()
{
	FILE *fp=NULL;
	int j=-1;

	if ( !(fp=fopen(CONF_FILE, "w+")) )
	{
		perror("fichier de configuration");
		gtk_main_quit();
		exit(0);
	}
	while (file[++j])
	{
		fputs(file[j], fp);
		putc('\n', fp);
	}
	fclose(fp);
}
