#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "conf_ban.h"
#include "debug.h"
#include "conf.h"
#include "conf_ban_types.h"
#include "conf_win.h"
#include "div.h"
#include "mesg_box.h"


static void add_ban_click(GtkWidget *btn);
static void remove_ban_click(GtkWidget *btn);
static GtkWidget *get_banlist();
static void ban_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data);
static void add_column(GtkTreeView *tree);
/*static void inverse_ban_list(GtkTreeViewColumn *col, GtkTreeView *tree);*/
static void insert_ban(GtkTreeView *tree, Tban *ban);
static Tban *new_ban(const char *ip);
static Tban *get_ban_selected();
static void remove_ban(Tban *ban);


static GtkWidget *tbl;
static GtkWidget *tree;


GtkWidget *get_ban()
{
	GtkWidget *lbl, *btn_add_ban, *btn_remove_ban;

	ban_list = NULL;

	lbl = gtk_label_new("You can see the ban list :");
	btn_add_ban = gtk_button_new_from_stock(GTK_STOCK_ADD);
	btn_remove_ban = gtk_button_new_from_stock(GTK_STOCK_REMOVE);

	tbl = gtk_table_new(3, 2, FALSE);
	gtk_table_attach(GTK_TABLE(tbl), lbl, 0, 2, 0, 1, GTK_EXPAND, 0, 0, 5);
	gtk_table_attach(GTK_TABLE(tbl), get_banlist(), 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 5);
	gtk_table_attach(GTK_TABLE(tbl), btn_add_ban, 0, 1, 2, 3, GTK_EXPAND, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), btn_remove_ban, 1, 2, 2, 3, GTK_EXPAND, 0, 5, 5);

	g_signal_connect(G_OBJECT(btn_add_ban), "clicked", G_CALLBACK(add_ban_click), NULL);
	g_signal_connect(G_OBJECT(btn_remove_ban), "clicked", G_CALLBACK(remove_ban_click), NULL);

	return (tbl);
}


static void add_ban_click(GtkWidget *btn)
{
	char *ip;

	ip = get_entry_dbox(GTK_WINDOW(get_conf_win()), "Add an IP to ban list", "IP : ");
	if ( ip && is_valid_ip(ip) )
	{
		new_ban(ip);
		free(ip);
		gtk_widget_show_all(GTK_WIDGET(tree));
	}
}


static void remove_ban_click(GtkWidget *btn)
{
	Tban *ban;

	ban = get_ban_selected();
	if (ban)
		remove_ban(ban);
}


static GtkWidget *get_banlist()
{
	int i;
	GtkWidget *sw;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	char **list_ban;

	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(sw, 150, 200);
	store = gtk_list_store_new(1, G_TYPE_STRING);
	model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
	tree = gtk_tree_view_new_with_model(model);
	g_object_unref(G_OBJECT(model));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	gtk_tree_selection_set_select_function(selection, (GtkTreeSelectionFunc) ban_selected, NULL, NULL);
	gtk_container_add(GTK_CONTAINER(sw), tree);
	add_column(GTK_TREE_VIEW(tree));

	list_ban= get_settings(BAN_BALISE);
	i=-1;
	if (list_ban)
		while(list_ban[++i])
			new_ban(list_ban[i]);

	return(sw);
}


static void add_column(GtkTreeView *tree)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Ban list :", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_reorderable(column, TRUE);
	gtk_tree_view_set_rules_hint(tree, TRUE);
	/*gtk_tree_view_column_set_clickable(column, TRUE);
	gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
	gtk_tree_view_column_set_clickable(column, FALSE);
	gtk_tree_view_column_set_sort_indicator(column, FALSE);
	gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
	gtk_tree_view_column_clicked(column);
	/*g_signal_connect(column, "clicked", G_CALLBACK(inverse_ban_list), tree);*/
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
}
	

static void insert_ban(GtkTreeView *tree, Tban *ban)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_append(GTK_LIST_STORE(model), &ban->iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &ban->iter, 0, ban->ip, -1);
	gtk_widget_show_all(get_conf_win());
}


static Tban *new_ban(const char *ip)
{
	Tban *ban;

	ban = (Tban *) malloc(sizeof(Tban));
	memset(ban, 0, sizeof(Tban));
	ban->ip = strdup(ip);
	ban->selected = FALSE;
	ban->next = ban_list;
	ban_list = ban;
	insert_ban(GTK_TREE_VIEW(tree), ban);
	return(ban);
}

static void ban_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data)
{
	GtkTreeIter iter;
	char *ip;
	Tban *ban;

	if (!selected)
	{
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 0, &ip, -1);
		ban = ban_list;
		while (ban)
		{
			if (!strcasecmp(ban->ip, ip))
				ban->selected = TRUE;
			else
				ban->selected = FALSE;
			ban = ban->next;
		}
		free(ip);
	}
	gtk_widget_show_all(get_conf_win());
}


static Tban *get_ban_selected()
{
	Tban *ban;

	ban = ban_list;
	while (ban)
	{
		if (ban->selected)
			return(ban);
		ban = ban->next;
	}
	return(NULL);
}


static void remove_ban(Tban *ban)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;
	Tban *fst_ban, *prec_ban=NULL;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_remove(GTK_LIST_STORE(model), &ban->iter);

	fst_ban = ban_list;
	while (fst_ban)
	{
		if (fst_ban==ban)
		{
			if (prec_ban==NULL)
				ban_list = ban->next;
			else
				prec_ban->next = ban->next;
			if ( !prec_ban && !ban->next)
				empty_conf(BAN_BALISE);
			if (ban->ip)
				free(ban->ip);
			free(ban);
			gtk_widget_show_all(GTK_WIDGET(tbl));
			return;
		}
		prec_ban = fst_ban;
		fst_ban = fst_ban->next;
	}
	gtk_widget_show_all(get_conf_win());
}



void save_ban()
{
	Tban *ban;
	gboolean more_ban;

	more_ban = FALSE;
	ban = ban_list;
	while (ban)
	{
		put_one_setting_str_of_multiple(ban->ip);
		more_ban = TRUE;
		ban = ban->next;
	}
	if (more_ban)
		put_multiple_setting(BAN_BALISE);
	else
		empty_conf(BAN_BALISE);
}
