#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include "conf_conf.h"
#include "debug.h"
#include "conf.h"

#define ICONPATH_LBL "Icon's path : "
#define ICONPATH_BTN "Edit"
#define HISTORY_LBL "Number of lines : "
#define PORT_LBL "Port number : "
#define MAXUSER_LBL "Number of users allowed simultaneous : "

#define ENTRY_LENGTH 6

typedef struct _Tconf Tconf;
typedef struct _Tconf_gtk Tconf_gtk;
typedef struct _Tconf_val Tconf_val;

struct _Tconf_gtk {
	GtkWidget *iconpath_label, *iconpath_entry, *iconpath_button, *file_select;
	GtkWidget *history_label, *history_entry;
	GtkWidget *port_label, *port_entry;
	GtkWidget *maxuser_label, *maxuser_entry;
};

struct _Tconf_val {
	char *iconpath;
	int history;
	int port;
	int maxuser;
};

struct _Tconf {
	Tconf_gtk gtk;
	Tconf_val val;
};


static void load_conf();
static void init_conf_val();
static void load_conf_val();
static void load_conf_gtk();
static void iconpath_click(GtkWidget *btn);
static void iconpath_chosen(GtkFileSelection *file_select, gpointer data);

Tconf conf;

GtkWidget *get_conf()
{
	GtkWidget *tbl;

	load_conf();

	tbl = gtk_table_new(4, 3, FALSE);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.iconpath_label, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.iconpath_entry, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.iconpath_button, 2, 3, 0, 1, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.history_label, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.history_entry, 2, 3, 1, 2, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.port_label, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.port_entry, 2, 3, 2, 3, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.maxuser_label, 0, 2, 3, 4, GTK_EXPAND | GTK_FILL, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), conf.gtk.maxuser_entry, 2, 3, 3, 4, 0, 0, 5, 5);

	g_signal_connect(G_OBJECT(conf.gtk.iconpath_button), "clicked", G_CALLBACK(iconpath_click), NULL);

	return(tbl);
}


static void load_conf()
{
	init_conf_val();
	load_conf_val();
	load_conf_gtk();
}

static void init_conf_val()
{
	conf.val.iconpath = NULL;
	conf.val.history = 0;
	conf.val.port = 0;
	conf.val.maxuser = 0;
}


static void load_conf_val()
{
	if (get_setting(ICON_BALISE))
		conf.val.iconpath = strdup(get_setting(ICON_BALISE));
	if (get_setting(HISTORY_BALISE))
		conf.val.history = atoi(get_setting(HISTORY_BALISE));
	if (get_setting(PORT_BALISE))
		conf.val.port = atoi(get_setting(PORT_BALISE));
	if (get_setting(MAXUSER_BALISE))
		conf.val.maxuser = atoi(get_setting(MAXUSER_BALISE));
}


static void load_conf_gtk()
{
	char temp[5];

	conf.gtk.iconpath_label = gtk_label_new(ICONPATH_LBL);
	conf.gtk.iconpath_entry = gtk_label_new(conf.val.iconpath);
	conf.gtk.iconpath_button = gtk_button_new_with_label(ICONPATH_BTN);

	conf.gtk.history_label = gtk_label_new(HISTORY_LBL);
	conf.gtk.history_entry = gtk_entry_new_with_max_length(ENTRY_LENGTH);
	sprintf(temp, "%d", conf.val.history);
	gtk_entry_set_text(GTK_ENTRY(conf.gtk.history_entry), temp);

	conf.gtk.port_label = gtk_label_new(PORT_LBL);
	conf.gtk.port_entry = gtk_entry_new_with_max_length(ENTRY_LENGTH);
	sprintf(temp, "%d", conf.val.port);
	gtk_entry_set_text(GTK_ENTRY(conf.gtk.port_entry), temp);

	conf.gtk.maxuser_label = gtk_label_new(MAXUSER_LBL);
	conf.gtk.maxuser_entry = gtk_entry_new_with_max_length(ENTRY_LENGTH);
	sprintf(temp, "%d", conf.val.maxuser);
	gtk_entry_set_text(GTK_ENTRY(conf.gtk.maxuser_entry), temp);
}


void save_conf()
{
	if (conf.val.iconpath)
	{
		put_setting_str(ICON_BALISE, conf.val.iconpath);
		free(conf.val.iconpath);
	}
	conf.val.history = atoi(gtk_entry_get_text(GTK_ENTRY(conf.gtk.history_entry)));
	if (conf.val.history > 0)
		put_setting_int(HISTORY_BALISE, conf.val.history);
	conf.val.port = atoi(gtk_entry_get_text(GTK_ENTRY(conf.gtk.port_entry)));
	if (conf.val.port > 0)
		put_setting_int(PORT_BALISE, conf.val.port);
	conf.val.maxuser = atoi(gtk_entry_get_text(GTK_ENTRY(conf.gtk.maxuser_entry)));
	if (conf.val.maxuser > 0)
		put_setting_int(MAXUSER_BALISE, conf.val.maxuser);
}


static void iconpath_click(GtkWidget *btn)
{
	conf.gtk.file_select = gtk_file_selection_new("Please, select the path for gtkftpd's icon");
	g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(conf.gtk.file_select)->ok_button), "clicked", 
			G_CALLBACK(iconpath_chosen), (gpointer) conf.gtk.file_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(conf.gtk.file_select)->ok_button), "clicked", 
			G_CALLBACK(gtk_widget_destroy), (gpointer) conf.gtk.file_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(conf.gtk.file_select)->cancel_button), "clicked", 
			G_CALLBACK(gtk_widget_destroy), (gpointer) conf.gtk.file_select);
	gtk_widget_show(conf.gtk.file_select);
}


static void iconpath_chosen(GtkFileSelection *file_select, gpointer data)
{
	conf.val.iconpath = strdup(gtk_file_selection_get_filename(GTK_FILE_SELECTION(conf.gtk.file_select)));
	gtk_label_set_text(GTK_LABEL(conf.gtk.iconpath_entry), conf.val.iconpath);
}
