#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "debug.h"
#include "conf.h"
#include "conf_perm.h"
#include "conf_perm_types.h"
#include "ftp_cmd.h"
#include "mesg_box.h"
#include "conf_win.h"

#ifndef _XOPEN_SOURCE_
#define _XOPEN_SOURCE_
#endif


static void load_perm(const Tconf_user *user2load);
static GtkWidget *get_listuser();
static void user_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data);
static void dir_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data);
static void add_column(GtkTreeView *tree);
static void add_dir_column(GtkTreeView *tree);
/*static void inverse_listuser(GtkTreeViewColumn *col, GtkTreeView *tree);*/
static void insert_user(GtkTreeView *tree, Tconf_user *user);
static void add_user_click(GtkWidget *btn);
static void del_user_click(GtkWidget *btn);
static void add_dir_click(GtkWidget *btn);
static void del_dir_click(GtkWidget *btn);
static void recurse_click(GtkWidget *btn);
static Tconf_user *new_user(const char *name);
static Tconf_user *new_anonymous_user(const char *name);
static void new_dir(const char *path, const Tright *rights);
static void fill_list_conf_user();
static Tconf_user *get_user_selected();
static Tperm_dir *get_dir_selected();
static void toggle_allow_anonymous(GtkWidget *togl, gpointer data);
static void toggle_dir_cwd(GtkWidget *cwd, gpointer data);
static void toggle_dir_list(GtkWidget *list, gpointer data);
static void toggle_dir_read(GtkWidget *read, gpointer data);
static void toggle_dir_write(GtkWidget *write, gpointer data);
static void remove_user(Tconf_user *user);
static void remove_user_by_name(const char *name);
static GtkWidget *init_empty_permtree();
static void reinit_empty_permtree();
static void init_permtree();
static void update_permtree();
static void init_Tperm(Tperm *perm);
static void init_Tperm_tree(Tperm_tree *tree);
static void free_Tconf_user(Tconf_user *user);
static void insert_dir(GtkTreeView *tree, Tperm_dir *dir);
static void vide_list_dir(GtkTreeView *tree);
static void update_permdir(Tperm_dir *dir);
static void dir_chosen(GtkFileSelection *file_select, gpointer data);
static void rootdir_chosen(GtkFileSelection *file_select, gpointer data);
static void init_Tperm_dir(Tperm_dir *dir);
static gboolean ever_exist_dir(const char *path);
static void enable_permdir(gboolean enable);
static void free_Tperm_dir(Tperm_dir *dir);
static void remove_dir(Tperm_dir *dir);
static void modify_rootdir_click(GtkWidget *btn);
static void modify_password_click(GtkWidget *btn);
static void verif_dirs_in_rootdir(Tconf_user *user, const char *rootdir);
static void do_recurse_rights(const char *path, const Tright rights, gboolean first);


static GtkWidget *tree, *right_tbl;
static Tperm *perm;



static void fill_list_conf_user()
{
	char **users;
	int i=-1;
	
	users = get_settings(USERS_BALISE);
	if (users)
		while (users[++i])
			new_user(users[i]);
}


static Tconf_user *new_anonymous_user(const char *name)
{
	Tconf_user *user;

	user = new_user(name);
	user->anonymous = TRUE;
	return(user);
}


static Tconf_user *new_user(const char *name)
{
	Tconf_user *user;

	user = (Tconf_user *) malloc(sizeof(Tconf_user));
	memset(user, 0, sizeof(Tconf_user));
	user->name = strdup(name);
	user->selected = FALSE;
	user->anonymous = FALSE;
	user->next = list_conf_user;
	list_conf_user = user;
	insert_user(GTK_TREE_VIEW(tree), user);
	return(user);
}

static void new_dir(const char *path, const Tright *rights)
{
	Tperm_dir *dir;
	Tconf_user *user;

	dir = (Tperm_dir *) malloc(sizeof(Tperm_dir));
	init_Tperm_dir(dir);
	dir->path = strdup(path);
	if (rights)
	{
		dir->rights.cwd = rights->cwd;
		dir->rights.list = rights->list;
		dir->rights.read = rights->read;
		dir->rights.write = rights->write;
	}
	else
		dir->path[strlen(dir->path)-1] = 0;
	user = get_user_selected();
	dir->next = user->perm->list_dir;
	user->perm->list_dir = dir;
	insert_dir(GTK_TREE_VIEW(perm->perm_tree->tree),dir);
}



GtkWidget *get_perm()
{
	GtkWidget *togl;

	list_conf_user = NULL;
	perm = NULL;
	right_tbl = gtk_table_new(2, 2, FALSE);
	togl = gtk_check_button_new_with_label("Allow anonymous user");
	gtk_table_attach(GTK_TABLE(right_tbl), togl, 0, 1, 0, 1, GTK_EXPAND, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(right_tbl), get_listuser(), 0, 1, 1, 2, GTK_EXPAND | GTK_FILL,
			GTK_EXPAND | GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(right_tbl), init_empty_permtree(), 1, 2, 1, 2, GTK_EXPAND | GTK_FILL,
			GTK_EXPAND | GTK_FILL, 5, 5);

	if (atoi(get_setting(ANONYMOUS_BALISE))==1)
	{
		new_anonymous_user("anonymous");
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(togl), TRUE);
	}

	g_signal_connect(GTK_OBJECT(togl), "toggled", G_CALLBACK(toggle_allow_anonymous), NULL);

	load_perm(NULL);

	return(right_tbl);
}


static GtkWidget *get_listuser()
{
	GtkWidget *tbl;
	GtkTreeModel *model;
	GtkTreeSelection *selection;
	GtkWidget *sw;
	GtkListStore *store;
	GtkWidget *add, *del;

	tbl = gtk_table_new(2, 2, FALSE);
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(sw, 150, 200);
	store = gtk_list_store_new(1, G_TYPE_STRING);
	model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
	tree = gtk_tree_view_new_with_model(model);
	g_object_unref(G_OBJECT(model));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	gtk_tree_selection_set_select_function(selection, (GtkTreeSelectionFunc) user_selected, NULL, NULL);
	gtk_container_add(GTK_CONTAINER(sw), tree);
	add_column(GTK_TREE_VIEW(tree));

	fill_list_conf_user();

	add = gtk_button_new_from_stock(GTK_STOCK_ADD);
	del = gtk_button_new_from_stock(GTK_STOCK_REMOVE);

	gtk_table_attach(GTK_TABLE(tbl), sw, 0, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), add, 0, 1, 1, 2, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(tbl), del, 1, 2, 1, 2, 0, 0, 5, 5);
	g_signal_connect(G_OBJECT(add), "clicked", G_CALLBACK(add_user_click), NULL);
	g_signal_connect(G_OBJECT(del), "clicked", G_CALLBACK(del_user_click), NULL);
	return(tbl);
}


static void add_user_click(GtkWidget *btn)
{
	char *name;
	Tconf_user *user;

	name = get_entry_dbox(GTK_WINDOW(get_conf_win()), "Add a user", "Name:");
	if (name)
	{
		user = new_user(name);
		load_perm(user);
		free(name);
		gtk_widget_show_all(GTK_WIDGET(tree));
	}
}


static void del_user_click(GtkWidget *btn)
{
	Tconf_user *user;
	GtkWidget *dlg;

	user = get_user_selected();
	if ( user )
	{
		if ( !strncasecmp(user->name, "anonymous", strlen(user->name)) )
		{
			dlg = gtk_message_dialog_new(GTK_WINDOW(get_conf_win()), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR,
					GTK_BUTTONS_OK, "Anonymous users are allowed, if you really want it desactivate anonymous login");
			gtk_dialog_run(GTK_DIALOG(dlg));
			gtk_widget_destroy(dlg);
		} else
			remove_user(user);
	}
}


static void add_dir_click(GtkWidget *btn)
{
	perm->file_select = gtk_file_selection_new("Please, select a directory");
	g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(perm->file_select)->ok_button), "clicked", 
			G_CALLBACK(dir_chosen), (gpointer) perm->file_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(perm->file_select)->ok_button), "clicked",
			G_CALLBACK(gtk_widget_destroy), (gpointer) perm->file_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(perm->file_select)->cancel_button), "clicked",
			G_CALLBACK(gtk_widget_destroy), (gpointer) perm->file_select);
	gtk_widget_show(perm->file_select);
}


static void dir_chosen(GtkFileSelection *select, gpointer data)
{
	const gchar *path;
	Tconf_user *user;
	struct stat st;

	path = gtk_file_selection_get_filename(GTK_FILE_SELECTION(perm->file_select));
	if (path[strlen(path)-1]!='/')
	{
		gtk_widget_destroy(perm->file_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "You have selected a file ! but we want a directory...");
		return;
	}
	if ( stat(path, &st) || !(st.st_mode & S_IFDIR) )
	{
		gtk_widget_destroy(perm->file_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "You have not selected a valid directory.");
		return;
	}
	user = get_user_selected();
	if (!user->perm->rootdir)
	{
		gtk_widget_destroy(perm->file_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "Please, specify a root directory before");
		return;
	}
	if ( strncasecmp(path, user->perm->rootdir, strlen(user->perm->rootdir)) )
	{
		gtk_widget_destroy(perm->file_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "The directory selected isn't in the root directory...");
		return;
	}
	if ( ever_exist_dir(path) )
	{
		gtk_widget_destroy(perm->file_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "The directory selected is ever in the list");
		return;
	}
	new_dir(path, NULL);
}



static gboolean ever_exist_dir(const char *path)
{
	Tperm_dir *dir;
	Tconf_user *user;

	user = get_user_selected();
	dir = user->perm->list_dir;
	while (dir)
	{
		if ( !strncmp(dir->path, path, strlen(path)-1) )
			return(TRUE);
		dir = dir->next;
	}
	return(FALSE);
}



static void del_dir_click(GtkWidget *btn)
{
	Tperm_dir *dir;

	dir = get_dir_selected();
	if (dir)
		remove_dir(dir);
	else
		error_message_box(GTK_WINDOW(get_conf_win()), "You have not selected any directory to remove");
	enable_permdir(FALSE);
}


static Tconf_user *get_user_selected()
{
	Tconf_user *user;

	user = list_conf_user;
	while (user)
	{
		if (user->selected)
			return(user);
		user = user->next;
	}
	return(NULL);
}

static Tperm_dir *get_dir_selected()
{
	Tperm_dir *dir;
	Tconf_user *user;

	user = get_user_selected();
	dir = user->perm->list_dir;
	while (dir)
	{
		if (dir->selected)
			return(dir);
		dir = dir->next;
	}
	return(NULL);
}


static void add_column(GtkTreeView *tree)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Utilisateurs :", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_reorderable(column, TRUE);
	gtk_tree_view_set_rules_hint(tree, TRUE);
	/*gtk_tree_view_column_set_clickable(column, TRUE);
	gtk_tree_view_column_set_sort_indicator(column, TRUE);*/
	gtk_tree_view_column_set_clickable(column, FALSE);
	gtk_tree_view_column_set_sort_indicator(column, FALSE);
	gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
	gtk_tree_view_column_clicked(column);
	/*g_signal_connect(column, "clicked", G_CALLBACK(inverse_listuser), tree);*/
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
}


static void add_dir_column(GtkTreeView *tree)
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Directories:", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_reorderable(column, TRUE);
	gtk_tree_view_set_rules_hint(tree, TRUE);
	gtk_tree_view_column_set_clickable(column, FALSE);
	gtk_tree_view_column_set_sort_indicator(column, FALSE);
	gtk_tree_view_column_clicked(column);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
}


static void insert_user(GtkTreeView *tree, Tconf_user *user)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_append(GTK_LIST_STORE(model), &user->iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &user->iter, 0, user->name, -1);
	gtk_widget_show_all(get_conf_win());
}


/*static void inverse_listuser(GtkTreeViewColumn *col, GtkTreeView *tree)
{
	GtkTreeModelSort *model_sort;
	GtkTreeModel *model;
	GtkTreeIter iter, iter_prec;
	int i, cptr=-1;
	char *val;
	char **users;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_tree_model_get_iter_first(model, &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 0, &val, -1);
	gtk_list_store_remove(GTK_LIST_STORE_MODEL(model), &iter);
	gtk_list_store_append(GTK_LIST_STORE_MODEL(model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, val, -1);
	iter_prec = iter;

	users = get_settings(USERS_BALISE);
	while (users[++cptr])
		;
	for (i=0;i<cptr-2;i++)
	{
		gtk_tree_model_get_iter_first(model, &iter);
		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 0, &val, -1);
		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
		gtk_list_store_insert_before(GTK_LIST_STORE(model), &iter, &iter_prec);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, val, -1);
		iter_prec = iter;
	}

	if (gtk_tree_view_column_get_sort_order(column)==GTK_SORT_ASCENDING)
		gtk_tree_view_column_set_sort_order(column, GTK_SORT_DESCENDING);
	else
		gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
}*/



static void user_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data)
{
	GtkTreeIter iter;
	char *name;
	Tconf_user *user;

	if (!selected)
	{
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 0, &name, -1);
		user = list_conf_user;
		while (user)
		{
			if (!strcasecmp(user->name, name))
			{	
				user->selected = TRUE;
				update_permtree();
				enable_permdir(FALSE);
			} else
				user->selected = FALSE;
			user = user->next;
		}
		free(name);
	}
	gtk_widget_show_all(get_conf_win());
}


static void toggle_allow_anonymous(GtkWidget *togl, gpointer data)
{
	Tconf_user *user;

	if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togl)) )
	{
		user = new_user("anonymous");
		load_perm(user);
		gtk_widget_show_all(GTK_WIDGET(tree));
	} else
		remove_user_by_name("anonymous");
}


static void remove_user_by_name(const char *name)
{
	Tconf_user *user;

	user = list_conf_user;
	while (user)
	{
		if ( !strncasecmp(name, user->name, strlen(name)) )
		{
			remove_user(user);
			return;
		}
		user = user->next;
	}

}

static void remove_user(Tconf_user *user)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;
	Tconf_user *fst_user, *prec_user=NULL;
	char *buf;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_remove(GTK_LIST_STORE(model), &user->iter);

	fst_user = list_conf_user;
	while(fst_user)
	{
			if (fst_user==user)
			{
				if (prec_user==NULL)
					list_conf_user = user->next;
				else
					prec_user->next = user->next;
				buf = malloc(strlen(user->name)+2);
				memset(buf, 0, strlen(user->name)+2);
				strncpy(buf, user->name, strlen(user->name));
				strncat(buf, "_", 1);
				empty_conf(buf);
				if (prec_user==NULL && user->next==NULL)
					empty_conf(USERS_BALISE);
				reinit_empty_permtree();
				free_Tconf_user(user);
				gtk_widget_show_all(right_tbl);
				return;
			}
			prec_user = fst_user;
			fst_user = fst_user->next;
	}
	gtk_widget_show_all(get_conf_win());
}


static void remove_dir(Tperm_dir *dir)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;
	Tconf_user *user;
	Tperm_dir *fst_dir, *prec_dir=NULL;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(perm->perm_tree->tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_remove(GTK_LIST_STORE(model), &dir->iter);

	user = get_user_selected();
	fst_dir = user->perm->list_dir;
	while (fst_dir)
	{
		if (fst_dir==dir)
		{
			if (prec_dir==NULL)
				user->perm->list_dir = dir->next;
			else
				prec_dir->next = dir->next;
			free_Tperm_dir(dir);
			gtk_widget_show_all(right_tbl);
			return;
		}
		prec_dir = fst_dir;
		fst_dir = fst_dir->next;
	}
	gtk_widget_show_all(get_conf_win());
}


static void free_Tconf_user(Tconf_user *user)
{
	if (user)
	{
		if (user->perm)
		{
			if (user->perm->rootdir)
				free(user->perm->rootdir);
			if (user->perm->pass)
				free(user->perm->pass);
			if (user->perm->list_dir)
				free(user->perm->list_dir);
			free(user->perm);
		}
		if (user->name)
			free(user->name);
		free(user);
	}
}


static void reinit_empty_permtree()
{
	if (!perm)
		return;
	if (perm->name_label)
		gtk_object_destroy(GTK_OBJECT(perm->name_label));
	if (perm->rootdir_entry)
		gtk_object_destroy(GTK_OBJECT(perm->rootdir_entry));
	if (perm->rootdir_lbl)
		gtk_object_destroy(GTK_OBJECT(perm->rootdir_lbl));
	if (perm->pass_lbl)
		gtk_object_destroy(GTK_OBJECT(perm->pass_lbl));
	if (perm->perm_tree && perm->perm_tree->cwd_radio)
		gtk_object_destroy(GTK_OBJECT(perm->perm_tree->cwd_radio));
	if (perm->perm_tree && perm->perm_tree->list_radio)
		gtk_object_destroy(GTK_OBJECT(perm->perm_tree->list_radio));
	if (perm->perm_tree && perm->perm_tree->write_radio)
		gtk_object_destroy(GTK_OBJECT(perm->perm_tree->write_radio));
	if (perm->perm_tree && perm->perm_tree->read_radio)
		gtk_object_destroy(GTK_OBJECT(perm->perm_tree->read_radio));
	if (perm->perm_tree && perm->perm_tree)
		gtk_object_destroy(GTK_OBJECT(perm->perm_tree->tree));
	if (perm->add_btn)
		gtk_object_destroy(GTK_OBJECT(perm->add_btn));
	if (perm->del_btn)
		gtk_object_destroy(GTK_OBJECT(perm->del_btn));
	if (perm->sw)
		gtk_object_destroy(GTK_OBJECT(perm->sw));
	if (perm->tbl)
		gtk_object_destroy(GTK_OBJECT(perm->tbl));
	gtk_table_attach(GTK_TABLE(right_tbl), init_empty_permtree(), 1, 2, 1, 2, GTK_EXPAND | GTK_FILL,
				GTK_EXPAND | GTK_FILL, 5, 5);
}


static GtkWidget *init_empty_permtree()
{
	GtkWidget *lbl;

	perm = malloc(sizeof(Tperm));
	init_Tperm(perm);
	perm->tbl = gtk_table_new(1, 1, FALSE);
	lbl = gtk_label_new("You should select a user to update his rights");
	gtk_table_attach(GTK_TABLE(perm->tbl), lbl, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 5, 5);
	return(perm->tbl);
}

static void init_permtree()
{
	GtkWidget *rootdir_lbl, *add_btn, *del_btn, *sw, *rootdir_btn, *pass_btn;
	GtkListStore *store;
	GtkTreeModel *model;
	GtkTreeSelection *selection;

	perm->empty = FALSE;
	gtk_object_destroy(GTK_OBJECT(perm->tbl));
	perm->tbl = gtk_table_new(3, 10, FALSE);
	perm->name_label = gtk_label_new("");
	rootdir_lbl = gtk_label_new("root directory : ");
	perm->rootdir_entry = gtk_label_new("");
	rootdir_btn = gtk_button_new_with_label("Edit...");
	gtk_label_set_justify(GTK_LABEL(perm->rootdir_entry), GTK_JUSTIFY_LEFT);
	pass_btn = gtk_button_new_with_label("Change password...");
	perm->perm_tree = malloc(sizeof(Tperm_tree));
	init_Tperm_tree(perm->perm_tree);
	perm->perm_tree->cwd_radio = gtk_check_button_new_with_label("CD");
	perm->perm_tree->list_radio = gtk_check_button_new_with_label("LIST");
	perm->perm_tree->write_radio = gtk_check_button_new_with_label("WRITE");
	perm->perm_tree->read_radio = gtk_check_button_new_with_label("READ");
	perm->perm_tree->recurse_btn = gtk_button_new_with_label("Recursive");
	enable_permdir(FALSE);
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(sw, 150, 200);
	store = gtk_list_store_new(1, G_TYPE_STRING);
	model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
	perm->perm_tree->tree = gtk_tree_view_new_with_model(model);
	g_object_unref(G_OBJECT(model));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(perm->perm_tree->tree));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	gtk_tree_selection_set_select_function(selection, (GtkTreeSelectionFunc) dir_selected, NULL, NULL);
	gtk_container_add(GTK_CONTAINER(sw), perm->perm_tree->tree);
	add_dir_column(GTK_TREE_VIEW(perm->perm_tree->tree));

	add_btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
	del_btn = gtk_button_new_from_stock(GTK_STOCK_REMOVE);

	gtk_table_attach(GTK_TABLE(perm->tbl), perm->name_label, 0, 3, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), rootdir_lbl, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->rootdir_entry, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND,
			GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), rootdir_btn, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), pass_btn, 0, 3, 2, 3, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), sw, 0, 2, 3, 9, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->perm_tree->cwd_radio, 2, 3, 3, 4, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->perm_tree->list_radio, 2, 3, 4, 5, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->perm_tree->write_radio, 2, 3, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->perm_tree->read_radio, 2, 3, 6, 7, GTK_FILL, GTK_FILL, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), perm->perm_tree->recurse_btn, 2, 3, 7, 8, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), add_btn, 0, 1, 9, 10, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(perm->tbl), del_btn, 1, 2, 9, 10, 0, 0, 5, 5);

	g_signal_connect(G_OBJECT(add_btn), "clicked", G_CALLBACK(add_dir_click), NULL);
	g_signal_connect(G_OBJECT(del_btn), "clicked", G_CALLBACK(del_dir_click), NULL);
	g_signal_connect(G_OBJECT(rootdir_btn), "clicked", G_CALLBACK(modify_rootdir_click), NULL);
	g_signal_connect(G_OBJECT(pass_btn), "clicked", G_CALLBACK(modify_password_click), NULL);
	g_signal_connect(G_OBJECT(perm->perm_tree->recurse_btn), "clicked", G_CALLBACK(recurse_click), NULL);
	g_signal_connect(GTK_OBJECT(perm->perm_tree->cwd_radio), "toggled", G_CALLBACK(toggle_dir_cwd), NULL);
	g_signal_connect(GTK_OBJECT(perm->perm_tree->list_radio), "toggled", G_CALLBACK(toggle_dir_list), NULL);
	g_signal_connect(GTK_OBJECT(perm->perm_tree->read_radio), "toggled", G_CALLBACK(toggle_dir_read), NULL);
	g_signal_connect(GTK_OBJECT(perm->perm_tree->write_radio), "toggled", G_CALLBACK(toggle_dir_write), NULL);
	
	gtk_table_attach(GTK_TABLE(right_tbl), perm->tbl, 1, 2, 0, 2, GTK_EXPAND | GTK_FILL, 
			GTK_EXPAND | GTK_FILL, 5, 5);
}

static void update_permtree()
{
	Tconf_user *user;
	Tperm_dir *dir;
	/*PangoAttrList *attr;*/

	if (perm->empty)
		init_permtree();
		
	user = get_user_selected();
	if (user)
	{
		if (user->name)
		{
			gtk_label_set_text(GTK_LABEL(perm->name_label), user->name);
			/*attr = pango_attr_list_new();																							// USER IN COLOR
			pango_attr_list_insert(attr, pango_attr_weight_new(PANGO_WEIGHT_BOLD));
			pango_attr_list_insert(attr, pango_attr_background_new(65535, 0, 0));
			gtk_label_set_attributes(GTK_LABEL(perm->name_label), attr);*/
		} else
			gtk_label_set_text(GTK_LABEL(perm->name_label), "");
		if (user->perm && user->perm->rootdir)
			gtk_label_set_text(GTK_LABEL(perm->rootdir_entry), user->perm->rootdir);
		else
			gtk_label_set_text(GTK_LABEL(perm->rootdir_entry), ""); 
		vide_list_dir(GTK_TREE_VIEW(perm->perm_tree->tree));
		if (user->perm)
			dir = user->perm->list_dir;
		else
			dir = NULL;
		while (dir)
		{
			insert_dir(GTK_TREE_VIEW(perm->perm_tree->tree), dir);
			dir = dir->next;
		}
		gtk_widget_show_all(right_tbl);
	}
}

static void vide_list_dir(GtkTreeView *tree)
{
	GtkTreeModelSort *model_sort;
	GtkTreeModel *model;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_clear(GTK_LIST_STORE(model));
}


static void insert_dir(GtkTreeView *tree, Tperm_dir *dir)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(tree)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_append(GTK_LIST_STORE(model), &dir->iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &dir->iter, 0, dir->path, -1);
	gtk_widget_show_all(GTK_WIDGET(tree));
}


static void dir_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data)
{
	GtkTreeIter iter;
	Tperm_dir *dir;
	char *rep;
	Tconf_user *user;

	if (!selected)
	{
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 0, &rep, -1);
		user = get_user_selected();
		dir = user->perm->list_dir;
		while (dir)
		{
			if (!strcasecmp(dir->path, rep))
			{
				dir->selected = TRUE;
				enable_permdir(TRUE);
				update_permdir(dir);
			} else
				dir->selected = FALSE;
			dir = dir->next;
		}
	}
	gtk_widget_show_all(right_tbl);
}


static void enable_permdir(gboolean enable)
{
	if (perm && perm->perm_tree && perm->perm_tree->cwd_radio)
	{
		gtk_widget_set_sensitive(perm->perm_tree->cwd_radio, enable);
		gtk_widget_set_sensitive(perm->perm_tree->list_radio, enable);
		gtk_widget_set_sensitive(perm->perm_tree->read_radio, enable);
		gtk_widget_set_sensitive(perm->perm_tree->write_radio, enable);
		if (!enable)
		{
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->cwd_radio), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->list_radio), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->read_radio), FALSE);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->write_radio), FALSE);
		}
	}
}

static void update_permdir(Tperm_dir *dir)
{
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->cwd_radio), dir->rights.cwd);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->list_radio), dir->rights.list);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->read_radio), dir->rights.read);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(perm->perm_tree->write_radio), dir->rights.write);
}


static void load_perm(const Tconf_user *user2load)
{
	char *balise;
	char **dir, **rights;
	Tperm_dir *perm_dir;
	Tconf_user *user;
	int i, j;

	user = list_conf_user;
	while (user)
	{
		if (!user2load || user2load==user)
		{
			i=j=-1;
			user->perm = malloc(sizeof(Tperm_user));
			memset(user->perm, 0, sizeof(Tperm_user));
			user->perm->list_dir = NULL;
			balise = malloc(strlen(ROOTDIR_PART_BALISE)+strlen(user->name)+1);
			memset(balise, 0, strlen(ROOTDIR_PART_BALISE)+strlen(user->name)+1);
			strncpy(balise, user->name, strlen(user->name));
			strncat(balise, ROOTDIR_PART_BALISE, strlen(ROOTDIR_PART_BALISE));
			if (get_setting(balise))
				user->perm->rootdir = strdup(get_setting(balise));
			else
				user->perm->rootdir = NULL;
			free(balise);
			if (!user->anonymous)
			{
				balise = malloc(strlen(PASS_PART_BALISE)+strlen(user->name)+1);
				memset(balise, 0, strlen(PASS_PART_BALISE)+strlen(user->name)+1);
				strncpy(balise, user->name, strlen(user->name));
				strncat(balise, PASS_PART_BALISE, strlen(PASS_PART_BALISE));
				if (get_setting(balise))
					user->perm->pass = strdup(get_setting(balise));
				else
					user->perm->pass = NULL;
				free(balise);
			}
			balise = malloc(strlen(LIMITDIRECTORY_BALISE)+strlen(user->name)+1);
			memset(balise, 0, strlen(LIMITDIRECTORY_BALISE)+strlen(user->name)+1);
			strncpy(balise, user->name, strlen(user->name));
			strncat(balise, LIMITDIRECTORY_BALISE, strlen(LIMITDIRECTORY_BALISE));
			dir = get_settings(balise);
			free(balise);
			if (dir)
			{
				i=-1;
				while (dir[++i])
				{
					perm_dir = malloc(sizeof(Tperm_dir));
					init_Tperm_dir(perm_dir);
					perm_dir->path = strdup(dir[i]);
					balise = malloc(strlen(dir[i])+strlen(user->name)+strlen(PERM_PART_BALISE)+1);
					memset(balise, 0, strlen(dir[i])+strlen(user->name)+strlen(PERM_PART_BALISE)+1);
					strncpy(balise, user->name, strlen(user->name));
					strncat(balise, PERM_PART_BALISE, strlen(PERM_PART_BALISE));
					strncat(balise, dir[i], strlen(dir[i]));
					rights = get_settings(balise);
					free(balise);
					if (rights)
					{
						perm_dir->rights.cwd = TRUE;
						perm_dir->rights.list = TRUE;
						perm_dir->rights.read = TRUE;
						perm_dir->rights.write = TRUE;
						j=-1;
						while (rights[++j])
						{
							if (!strcasecmp(rights[j], "CWD"))
								perm_dir->rights.cwd = FALSE;
							if (!strcasecmp(rights[j], "LIST"))
								perm_dir->rights.list = FALSE;
							if (!strcasecmp(rights[j], "STOR"))
								perm_dir->rights.write = FALSE;
							if (!strcasecmp(rights[j], "GET"))
								perm_dir->rights.read = FALSE;
						}
						while (j>0)
							free(rights[--j]);
						free(rights);
						perm_dir->next = user->perm->list_dir;
						user->perm->list_dir = perm_dir;
					}
				}
				while (i>0)
					free(dir[--i]);
				free(dir);
			} else
				user->perm->list_dir = NULL;
		}
		user = user->next;
	}
}


gboolean save_perm()
{
	Tconf_user *user;
	Tperm_dir *dirs;
	char *balise;
	int i;
	gboolean arg, anon;
	gboolean more_user;
	int cptr;
	char *warn;
	char warn1[] = "Be carreful the user named ";
	char warn2[] = " has full (read, write, ...) access to his root directory";

	/* on commence par ajouter les noms */
	user = list_conf_user;
	more_user = FALSE;
	while (user)
	{
		if (strncmp(user->name, "anonymous", strlen("anonymous")))
		{
			put_one_setting_str_of_multiple(user->name);
			more_user = TRUE;
		}
		user = user->next;
	}
	if (more_user)
		put_multiple_setting(USERS_BALISE);
	else
		empty_conf(USERS_BALISE);

	/* Anonymous login allowed ? */
	user = list_conf_user;
	anon = FALSE;
	while (user)
	{
		if	(!strncmp(user->name, "anonymous", strlen("anonymous")))
			anon = TRUE;
		user = user->next;
	}
	if (anon)
		put_setting_int(ANONYMOUS_BALISE, 1);
	else
		put_setting_int(ANONYMOUS_BALISE, 0);
	
	user = list_conf_user;
	while (user)
	{
		if (user->perm)
		{
			if (!user->perm->rootdir)
			{
				error_message_box(GTK_WINDOW(get_conf_win()), "You have not specify any rootdir for the new user");
				return (FALSE);
			}
			put_setting_str_and_deb_balise(user->name, ROOTDIR_PART_BALISE, user->perm->rootdir);
			if (strncmp(user->name, "anonymous", strlen("anonymous")))
			{
				if (!user->perm->pass)
				{
					error_message_box(GTK_WINDOW(get_conf_win()), "You have not specify any password for the new user");
					return (FALSE);
				}
				put_setting_str_and_deb_balise(user->name, PASS_PART_BALISE, user->perm->pass);
			}
			if (user->perm->list_dir)
			{
				/* utilisateurs avec tous les droits ? */
				dirs = user->perm->list_dir;
				cptr=0;
				while (dirs)
				{
					cptr++;
					dirs = dirs->next;
				}
				if (!cptr)
				{
					warn = malloc(strlen(warn1)+strlen(warn2)+strlen(user->name));
					strncpy(warn, warn1, strlen(warn1));
					strncat(warn, user->name, strlen(user->name));
					strncat(warn ,warn2, strlen(warn2));
					warning_message_box(GTK_WINDOW(get_conf_win()), warn);
					free(warn);
				}

				/* balise LIMITDIRECTORY contient les repertoires avec des autorisations speciales */
				dirs = user->perm->list_dir;
				while (dirs)
				{
					if (!dirs->rights.cwd || !dirs->rights.list || !dirs->rights.read
							|| !dirs->rights.write)
						put_one_setting_str_of_multiple(dirs->path);
					dirs = dirs->next;
				}
				balise = malloc(strlen(LIMITDIRECTORY_BALISE)+strlen(user->name)+1);
				memset(balise, 0, strlen(LIMITDIRECTORY_BALISE)+strlen(user->name)+1);
				strncpy(balise, user->name, strlen(user->name));
				strncat(balise, LIMITDIRECTORY_BALISE, strlen(LIMITDIRECTORY_BALISE));
				put_multiple_setting(balise);
				free(balise);
				/* une balise pour toutes les regles */
				dirs = user->perm->list_dir;
				while (dirs)
				{
					arg = FALSE;
					if (!dirs->rights.cwd)
					{
						put_one_setting_str_of_multiple("CWD");
						arg = TRUE;
					}
					if (!dirs->rights.list)
					{
						put_one_setting_str_of_multiple("LIST");
						arg = TRUE;
					}
					if (!dirs->rights.read)
					{
						i=-1;
						while (ftp_cmd_read[++i])
							put_one_setting_str_of_multiple(ftp_cmd_read[i]);
						arg = TRUE;
					}
					if (!dirs->rights.write)
					{
						i=-1;
						while (ftp_cmd_write[++i])
							put_one_setting_str_of_multiple(ftp_cmd_write[i]);
						arg = TRUE;
					}
					if (arg)
					{
						balise = malloc(strlen(user->name)+strlen(PERM_PART_BALISE)+strlen(dirs->path)+1);
						memset(balise, 0, strlen(user->name)+strlen(PERM_PART_BALISE)+strlen(dirs->path)+1);
						strncpy(balise, user->name, strlen(user->name));
						strncat(balise, PERM_PART_BALISE, strlen(PERM_PART_BALISE));
						strncat(balise, dirs->path, strlen(dirs->path));
						put_multiple_setting(balise);
						free(balise);
					} else {
						/* un repertoire a ete ajoutee, mais pas de limitation de droits */
						error_message_box(GTK_WINDOW(get_conf_win()), "Vous avez ajoute un repertoire sans en limiter les droits, il"
								"a donc ete supprime de votre liste");
					}
					dirs = dirs->next;
				}
			}
			else
			{
				warn = malloc(strlen(warn1)+strlen(warn2)+strlen(user->name));
				strncpy(warn, warn1, strlen(warn1));
				strncat(warn, user->name, strlen(user->name));
				strncat(warn ,warn2, strlen(warn2));
				warning_message_box(GTK_WINDOW(get_conf_win()), warn);
				free(warn);
			}
		}
		user = user->next;
	}

	return (TRUE);
}


static void init_Tperm(Tperm *perm)
{
	perm->tbl = NULL;
	perm->name_label = NULL;
	perm->rootdir_entry = NULL;
	perm->rootdir_lbl = NULL;
	perm->pass_lbl = NULL;
	perm->add_btn = NULL;
	perm->del_btn = NULL;
	perm->sw = NULL;
	perm->empty = TRUE;
	perm->file_select = NULL;
	perm->rootdir_select = NULL;
}

static void init_Tperm_tree(Tperm_tree *tree)
{
	tree->tree = NULL;
	tree->cwd_radio = NULL;
	tree->list_radio = NULL;
	tree->write_radio = NULL;
	tree->read_radio = NULL;
	tree->recurse_btn = NULL;
}

static void init_Tperm_dir(Tperm_dir *dir)
{
	dir->path = NULL;
	dir->rights.cwd = TRUE;
	dir->rights.list = TRUE;
	dir->rights.read = TRUE;
	dir->rights.write = TRUE;
	dir->selected = FALSE;
	dir->next = NULL;
}


static void toggle_dir_cwd(GtkWidget *cwd, gpointer data)
{
	Tperm_dir *dir;

	dir = get_dir_selected();
	if (dir && GTK_WIDGET_IS_SENSITIVE(cwd))
	{
		if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cwd)) )
			dir->rights.cwd = TRUE;
		else
			dir->rights.cwd = FALSE;
		update_permdir(dir);
		gtk_widget_show_all(right_tbl);
	}
}


static void toggle_dir_list(GtkWidget *list, gpointer data)
{
	Tperm_dir *dir;

	dir = get_dir_selected();
	if (dir && GTK_WIDGET_IS_SENSITIVE(list))
	{
		if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(list)) )
			dir->rights.list = TRUE;
		else
			dir->rights.list = FALSE;
		update_permdir(dir);
		gtk_widget_show_all(right_tbl);
	}
}


static void toggle_dir_read(GtkWidget *read, gpointer data)
{
	Tperm_dir *dir;

	dir = get_dir_selected();
	if (dir && GTK_WIDGET_IS_SENSITIVE(read))
	{
		if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(read)) )
			dir->rights.read = TRUE;
		else
			dir->rights.read = FALSE;
		update_permdir(dir);
		gtk_widget_show_all(right_tbl);
	}
}

static void toggle_dir_write(GtkWidget *write, gpointer data)
{
	Tperm_dir *dir;

	dir = get_dir_selected();
	if (dir && GTK_WIDGET_IS_SENSITIVE(write))
	{
		if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(write)) )
			dir->rights.write = TRUE;
		else
			dir->rights.write = FALSE;
		update_permdir(dir);
		gtk_widget_show_all(right_tbl);
	}
}


static void free_Tperm_dir(Tperm_dir *dir)
{
	free(dir->path);
	free(dir);
}


static void modify_rootdir_click(GtkWidget *btn)
{

	/*user = get_user_selected();
	user->perm->rootdir = get_entry_dbox("Specify a new root directory", "root directory :");
	printf("rootdir=%s\n", user->perm->rootdir);
	update_permtree();*/
	perm->rootdir_select = gtk_file_selection_new("Please, select your root directory");
	g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(perm->rootdir_select)->ok_button), "clicked", 
			G_CALLBACK(rootdir_chosen), (gpointer) perm->rootdir_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(perm->rootdir_select)->ok_button), "clicked",
			G_CALLBACK(gtk_widget_destroy), (gpointer) perm->rootdir_select);
	g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(perm->rootdir_select)->cancel_button), "clicked",
			G_CALLBACK(gtk_widget_destroy), (gpointer) perm->rootdir_select);
	gtk_widget_show(perm->rootdir_select);
}

static void rootdir_chosen(GtkFileSelection *file_select, gpointer data)
{
	const gchar *dir;
	Tconf_user *user;

	dir = gtk_file_selection_get_filename(GTK_FILE_SELECTION(perm->rootdir_select));
	if (dir[strlen(dir)-1]!='/')
	{
		gtk_widget_destroy(perm->rootdir_select);
		error_message_box(GTK_WINDOW(get_conf_win()), "You have selected a file ! but we want a directory...");
		return;
	}
	gtk_widget_destroy(perm->rootdir_select);
	user = get_user_selected();
	verif_dirs_in_rootdir(user, dir);
	free(user->perm->rootdir);
	user->perm->rootdir = strdup(dir);
	update_permtree();
}


static void verif_dirs_in_rootdir(Tconf_user *user, const char *rootdir)
{
	Tperm_dir *dir;
	gboolean warn=FALSE;

	if (!user->perm)
		return;
	dir = user->perm->list_dir;
	while (dir)
	{
		if ( strncmp(rootdir, dir->path, strlen(rootdir)-1) )
		{
			if (!warn)
			{
				if (!warning_message_box(GTK_WINDOW(get_conf_win()), "some of your dir are not in this root directory, they will be deleted"))
					return;
				warn = TRUE;
			}
			remove_dir(dir);
			dir = user->perm->list_dir;
			continue;
		}
		dir = dir->next;
	}
}


static void modify_password_click(GtkWidget *btn)
{
	Tconf_user *user;
	char *pass_tmp;

	user = get_user_selected();
	pass_tmp = get_entry_dbox(GTK_WINDOW(get_conf_win()), "Specify a new password", "New password :");
	user->perm->pass = crypt(pass_tmp, "4Z");																	// Hum ...
	free(pass_tmp);
}



static void recurse_click(GtkWidget *btn)
{
	Tperm_dir *perm_dir;
	Tright rights;

	perm_dir = get_dir_selected();
	if (!perm_dir)
	{
		error_message_box(GTK_WINDOW(get_conf_win()), "You have to specify a directory to make the rights recursives.");
		return;
	}
	do_recurse_rights(perm_dir->path, rights, TRUE);
}


/*
** Recursive function which make the rights for all the subdirectories
*/
static void do_recurse_rights(const char *path, const Tright rights, gboolean first)
{
	DIR *dir;
	struct dirent *entr;
	char *path_is_it_dir;
	struct stat st;

	if ( !(dir=opendir(path)) )
		return;
	while ( (entr=readdir(dir)) )
	{
		if (entr->d_name[0]=='.')
			continue;
		path_is_it_dir = malloc(strlen(path)+strlen(entr->d_name)+2);
		memset(path_is_it_dir, 0, strlen(path)+strlen(entr->d_name)+2);
		strncpy(path_is_it_dir, path, strlen(path));
		strncat(path_is_it_dir, "/", 1);
		strncat(path_is_it_dir, entr->d_name, strlen(entr->d_name));
		if ( !stat(path_is_it_dir, &st) && (st.st_mode & S_IFDIR) )
			do_recurse_rights(path_is_it_dir, rights, FALSE);
		free(path_is_it_dir);
	}
	if (!first)
		new_dir(path, &rights);
}
