#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.h"
#include "text.h"
#include "debug.h"
#include "statusbar.h"
#include "div.h"
#include "gtkftpd.h"
#include "server.h"
#include "mesg_box.h"
#include "conf.h"
#include "s_ret.h"
#include "win.h"


typedef struct _gtk_obj gtk_obj;
typedef struct _gtk_btn gtk_btn;

struct _gtk_btn	{
	GtkWidget *btn_kick;
	GtkWidget *btn_ban;
	GtkWidget *btn_kickban;
	GtkWidget *btn_getip;
};

struct _gtk_obj {
	gtk_btn btns;
	GtkWidget *treeview;
	GtkWidget *tbl;
};

//static GtkWidget *treeview=NULL;
//static GtkWidget *tbl=NULL;
//static GtkWidget *btn_kick=NULL, *btn_ban=NULL, *btn_kickban=NULL, *btn_ip=NULL;

gtk_obj objs;

static GtkTreeModel *create_model();
static void init_user_list(Tuser *user);
static void desinit_user_list(Tuser *user);
static void init_user_rezo(Tuser *user, Tuser_nt *nt);
static void desinit_user_rezo(Tuser *user);
static gboolean add_column();
static void update_name(char *leaving_name);
static void row_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data);
static Tuser *find_user(const char *name);
static void inverse_sorting(GtkTreeViewColumn *column, gpointer data);
static void activate_selected(Tuser *user);
static char *find_adaptive_name(char *actual_name);
static void click_kick_user(GtkWidget *btn);
static void click_ban_user(GtkWidget *btn);
static void click_kickban_user(GtkWidget *btn);
static void click_getip_user(GtkWidget *btn);


gboolean init_user(gchar *name, Tuser_nt *nt)
{
	Tuser *user;
	char sb_deb[]="New user is coming: ";
	char *sb;		// free dans statusbar.c (enfin, j'espere)

	if (!name)
	{
		name = (char *) malloc(4);
		memset(name, 0, 4);
		strncpy(name, "New", 4);
		name = find_adaptive_name(name);
	} else  {
		name = find_adaptive_name(name);
		strup(name);
		sb = (char *) malloc(strlen(sb_deb)+strlen(name)+1);
		memset(sb, 0, strlen(sb_deb)+strlen(name)+1);
		strncpy(sb, sb_deb, strlen(sb_deb));
		strncat(sb, name, strlen(name)+1);
		update_statusbar(sb);
	}

	user = g_malloc(sizeof(Tuser));
	memset(user, 0, sizeof(Tuser));
	user->name = name;
	user->id = g_slist_length(list_user);
	user->text = NULL;
	user->list = NULL;
	user->rezo = NULL;
	init_user_text(user);			// initialise le text pour l'affichage dans "all"
	if (nt)
		init_user_rezo(user, nt);
	if (!user->id)
		draw_user_text(user);		// a son propre onglet
	else
		init_user_list(user);		// initialse tlm dans la liste sauf all
	list_user = g_slist_append(list_user, user);

	return(TRUE);
}



void desinit_user(Tuser *user, gboolean rezo)
{
	if (user->text->sw)
		desinit_user_text(user);
	if (user->list)
		desinit_user_list(user);
	if (rezo && user->rezo)
		desinit_user_rezo(user);
	g_slist_remove(list_user, user);
	free(user);
}


void desinit_users()
{
	Tuser *user;
	int i;
	
	for (i=g_slist_length(list_user);i>0;i--)
	{
		user = (Tuser *) (g_slist_nth(list_user, i-1))->data;
		if (user->rezo && user->rezo->sock>0)
		{
			ret(user->rezo, RET_537);
			close(user->rezo->sock);
		}
		desinit_user(user, TRUE);
	}
	list_user = NULL;
}


gboolean init_list()
{
  GtkTreeModel *model;
	GtkTreeSelection *selection;
	GtkWidget *sw=NULL;


	objs.btns.btn_kick = NULL;
	objs.btns.btn_ban = NULL;
	objs.btns.btn_kickban = NULL;
	objs.btns.btn_getip = NULL;
	objs.treeview = NULL;
	objs.tbl = NULL;
	objs.tbl = gtk_table_new(4, 2, FALSE);
	objs.btns.btn_kick = gtk_button_new_with_label("Kick");
	objs.btns.btn_ban = gtk_button_new_with_label("Ban");
	objs.btns.btn_kickban = gtk_button_new_with_label("Kick & Ban");
	objs.btns.btn_getip = gtk_button_new_with_label("get IP");
	sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_table_attach(GTK_TABLE(objs.tbl), objs.btns.btn_kick, 0, 1, 0, 1, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(objs.tbl), objs.btns.btn_ban, 1, 2, 0, 1, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(objs.tbl), objs.btns.btn_kickban, 2, 3, 0, 1, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(objs.tbl), objs.btns.btn_getip, 3, 4, 0, 1, 0, 0, 5, 5);
	gtk_table_attach(GTK_TABLE(objs.tbl), sw, 0, 4, 1, 2, GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 5);

	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC,
									GTK_POLICY_AUTOMATIC);
	gtk_widget_set_usize(sw, 180, 200);

	model = create_model();
	objs.treeview = gtk_tree_view_new_with_model(model);
	g_object_unref(G_OBJECT(model));
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(objs.treeview));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
	gtk_tree_selection_set_select_function(selection, (GtkTreeSelectionFunc) row_selected, NULL, NULL);
	gtk_container_add(GTK_CONTAINER(sw), objs.treeview);
	if (!add_column())
		return(FALSE);

	g_signal_connect(G_OBJECT(objs.btns.btn_kick), "clicked", G_CALLBACK(click_kick_user), NULL);
	g_signal_connect(G_OBJECT(objs.btns.btn_ban), "clicked", G_CALLBACK(click_ban_user), NULL);
	g_signal_connect(G_OBJECT(objs.btns.btn_kickban), "clicked", G_CALLBACK(click_kickban_user), NULL);
	g_signal_connect(G_OBJECT(objs.btns.btn_getip), "clicked", G_CALLBACK(click_getip_user), NULL);

	return(TRUE);
}


static void row_selected(GtkTreeSelection *selection, GtkTreeModel *model, GtkTreePath *path, gboolean selected, gpointer data)
{
	GtkTreeIter iter;
	char *name, *sb;
	char sb_deb[]="User in color: ";
	Tuser *user;

	if (!selected)
	{
		gtk_tree_model_get_iter(model, &iter, path);
		gtk_tree_model_get(model, &iter, 0, &name, -1);

		sb = (char *) malloc(strlen(sb_deb)+strlen(name)+1);
		memset(sb, 0, strlen(sb_deb)+strlen(name)+1);
		strncpy(sb, sb_deb, strlen(sb_deb));
		strncat(sb, name, strlen(name)+1);
		update_statusbar(sb);

		if ( !(user = find_user(name)) )
			return;
		free(name);
		activate_color(user);
		update_history_color(user);
		activate_selected(user);
	}
}


void desinit_list()
{
	if (objs.btns.btn_kick)
		gtk_widget_destroy(objs.btns.btn_kick);
	if (objs.btns.btn_ban)
		gtk_widget_destroy(objs.btns.btn_ban);
	if (objs.btns.btn_kickban)
		gtk_widget_destroy(objs.btns.btn_kickban);
	if (objs.btns.btn_getip)
		gtk_widget_destroy(objs.btns.btn_getip);
	if (objs.treeview)
		gtk_widget_destroy(objs.treeview);
	if (objs.tbl)
		gtk_widget_destroy(objs.tbl);
}


GtkWidget *get_list()
{
	return(objs.tbl);
}


static GtkTreeModel *create_model()
{
	GtkListStore *store;
	GtkTreeModel *model;

	store = gtk_list_store_new(1, G_TYPE_STRING);
	model = gtk_tree_model_sort_new_with_model(GTK_TREE_MODEL(store));
	return(GTK_TREE_MODEL(model));
}


static void init_user_list(Tuser *user)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;
	GtkTreeIter *sibling = NULL;
	GtkTreeIter child_iter;
	GtkTreeViewColumn *column;
	Tuser *user_a;
	int i, higher_ecart=500, tmp=0;
	gboolean asc = FALSE;

	column = gtk_tree_view_get_column(GTK_TREE_VIEW(objs.treeview), 0);
	if (gtk_tree_view_column_get_sort_order(column)==GTK_SORT_ASCENDING)
	{
		asc = TRUE;
		higher_ecart = -500;
	}
	for (i=0;i<g_slist_length(list_user);i++)
	{
		user_a = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (asc==TRUE)
		{
			if ( user_a->list && (tmp=str_alpha_cmp(user_a->name, user->name))<0 )
			{
				if (tmp>=higher_ecart)
				{
					higher_ecart = tmp;
					sibling = &user_a->list->iter;
				}
			}
		} else {
			if ( user_a->list && (tmp=str_alpha_cmp(user_a->name, user->name))>0 )
			{
				if (tmp<=higher_ecart)
				{
					higher_ecart = tmp;
					sibling = &user_a->list->iter;
				}
			}
		}
	}
	user->list = g_malloc(sizeof(Tuser_list));
	memset(user->list, 0, sizeof(Tuser_list));
	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(objs.treeview)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_insert_after(GTK_LIST_STORE(model), &child_iter, sibling);
	gtk_list_store_set(GTK_LIST_STORE(model), &child_iter, 0, user->name, -1);
	user->list->iter = child_iter;
	user->list->selected = FALSE;
}


static void desinit_user_list(Tuser *user)
{
	GtkTreeModel *model;
	GtkTreeModelSort *model_sort;

	update_name(user->name);
	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(objs.treeview)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_list_store_remove(GTK_LIST_STORE(model), &user->list->iter);
	free(user->list);
}



static void init_user_rezo(Tuser *user, Tuser_nt *nt)
{
	unsigned char *ip;
	int i;

	user->rezo = nt;
	if (nt->ip[0]==0)  {
		ip = (unsigned char *) &nt->addr->sin_addr;
		for (i=0;i<4;i++)
			user->rezo->ip[i] = *ip++;
	}
}


static void desinit_user_rezo(Tuser *user)
{
	if (user->rezo->sock>0)
		close(user->rezo->sock);
	free(user->rezo);
}

static gboolean add_column()
{
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;

	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Utilisateurs:", renderer, "text", 0, NULL);
	gtk_tree_view_column_set_reorderable(column, TRUE);
	gtk_tree_view_column_set_clickable(column, TRUE);
	gtk_tree_view_column_set_sort_indicator(column, TRUE);
	gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
	gtk_tree_view_column_clicked(column);
	g_signal_connect(column, "clicked", G_CALLBACK(inverse_sorting), NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(objs.treeview), column);

	return(TRUE);
}	


/*
 * Donne un nom auto-incremente: anonymous, anonymous[2]
 */
static char *find_adaptive_name(char *actual_name)
{
	int i, last_name_no=0, temp=0;
	char *new_name;
	Tuser *user;

	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if ( !strncasecmp(actual_name, user->name, strlen(actual_name)) )
		{
			if ( *(user->name+strlen(user->name)-1)==']'
					&& *(user->name+strlen(user->name)-3)=='[' )
			{
				sscanf(user->name+strlen(user->name)-3, "[%d]", &temp);
				if (temp>last_name_no)
					last_name_no = temp;
			} else
				last_name_no++;
		}
	}
	if (!last_name_no)
		return(actual_name);
	else
	{
		new_name = (char *) malloc(strlen(actual_name)+4);
		memset(new_name, 0, strlen(actual_name)+4);
		sprintf(new_name, "%s[%d]", actual_name, last_name_no+1);
		return(new_name);
	}
}


/*
 * Mise a jour de tous les noms car un s'en va
 */
static void update_name(char *leaving_name)
{
	/*int i;
	Tuser *user;

	if ( *(leaving_name+strlen(leaving_name)-1)==']'
			&& *(leaving_name+strlen(leaving_name)-3)=='[' )
	{
		TRACE;
		return;
	}
	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if ( !strncasecmp(leaving_name, user->name, strlen(leaving_name)-
	}*/
}

static Tuser *find_user(const char *name)
{
	Tuser *user;
	int i;

	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if ( !strcasecmp(user->name,name) )
			return(user);
	}
	fprintf(stderr, "Error: can't find user with this name: %s", name);
	return(NULL);
}


static void inverse_sorting(GtkTreeViewColumn *column, gpointer data)
{
	GtkTreeModelSort *model_sort;
	GtkTreeModel *model;
	GtkTreeIter iter, iter_prec;
	int i;
	char *val;

	model_sort = GTK_TREE_MODEL_SORT(gtk_tree_view_get_model(GTK_TREE_VIEW(objs.treeview)));
	model = gtk_tree_model_sort_get_model(model_sort);
	gtk_tree_model_get_iter_first(model, &iter);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 0, &val, -1);
	gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
	gtk_list_store_append(GTK_LIST_STORE(model), &iter);
	gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, val, -1);
	iter_prec = iter;
	for (i=0;i<g_slist_length(list_user)-2;i++)
	{
		gtk_tree_model_get_iter_first(model, &iter);
		gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, 0, &val, -1);
		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
		gtk_list_store_insert_before(GTK_LIST_STORE(model), &iter, &iter_prec);
		gtk_list_store_set(GTK_LIST_STORE(model), &iter, 0, val, -1);
		iter_prec = iter;
	}

	if (gtk_tree_view_column_get_sort_order(column)==GTK_SORT_ASCENDING)
		gtk_tree_view_column_set_sort_order(column, GTK_SORT_DESCENDING);
	else
		gtk_tree_view_column_set_sort_order(column, GTK_SORT_ASCENDING);
}


static void activate_selected(Tuser *user)
{
	Tuser *user_a;
	int i;

	user->list->selected = TRUE;
	for (i=0;i<g_slist_length(list_user);i++)
	{
		user_a = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user_a->list && user!=user_a)
			user_a->list->selected = FALSE;
	}
}


Tuser *get_selected_user()
{
	Tuser *user;
	int i;

	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user->list && user->list->selected)
			return(user);
	}
	return(NULL);
}



static void click_kick_user(GtkWidget *btn)
{
	Tuser *user;

	user = get_selected_user();
	if (user)
		kick_user(user);
	else
		error_message_box(GTK_WINDOW(get_win()), "You want to kick someone but nobody is selected !");	
}


static void click_ban_user(GtkWidget *btn)
{
	char *banned;
	char *new_banned;
	char ip_new_banned[16];
	Tuser *user;

	user = get_selected_user();
	if (user)
	{
		banned = get_setting(BAN_BALISE);
		sprintf(ip_new_banned, "%d.%d.%d.%d", user->rezo->ip[0], user->rezo->ip[1],
				user->rezo->ip[2], user->rezo->ip[3]);
		if (banned)
		{
			new_banned = malloc(strlen(banned)+17);
			memset(new_banned, 0, strlen(banned)+17);
			strncpy(new_banned, banned, strlen(banned));
			strncat(new_banned, " ", 1);
			strncat(new_banned, ip_new_banned, strlen(ip_new_banned));
		}
		else
		{
			new_banned = malloc(16);
			memset(new_banned, 0, 16);
			strncpy(new_banned, ip_new_banned, strlen(ip_new_banned));
		}
		put_setting_str(BAN_BALISE, new_banned);
	}
	else
		error_message_box(GTK_WINDOW(get_win()), "You want to kick someone but nobody is selected !");	
}


static void click_kickban_user(GtkWidget *btn)
{
	click_ban_user(NULL);
	click_kick_user(NULL);
}


static void click_getip_user(GtkWidget *btn)
{
	Tuser *user;
	char ip[32];

	user = get_selected_user();
	if (user)
	{
		sprintf(ip, "IP : %d.%d.%d.%d", user->rezo->ip[0], user->rezo->ip[1], user->rezo->ip[2],
				user->rezo->ip[3]);
		info_message_box(GTK_WINDOW(get_win()), ip);
	}
	else
		error_message_box(GTK_WINDOW(get_win()), "You want to get someone's ip but nobody is selected !");	
}
