#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/stat.h>
#include <time.h>
#include <pthread.h>
#include <signal.h>
#include <sched.h>
#include "user.h"
#include "list.h"
#include "debug.h"
#include "statusbar.h"
#include "conf.h"
#include "s_ret.h"
#include "server.h"
#include "text.h"
#include "sys_cmd.h"
#include "secu.h"
#include "s_req_cst.h"


#define _XOPEN_SOURCE_
#define MAXPATHLEN 512
#define OUT_BUF 1024
#define BUF_FILE 4096
#define UMASK_MKD 0755


static void post_auth_task(Tuser *user);
static gboolean find_home_directory(Tuser *user);
static void bad_lp(Tuser *user);
static void access_denied(Tuser *user);
static void bad_home_directory(Tuser *user, char *path);
static unsigned char *getlocalip(int sock);
static void passive_send(const Tuser *user, char *buffer);
static void active_send(const Tuser *user, char *buffer);
static char *toutenun(char **buffer);
static char *arg2path(Tuser *user, const char *arg);
static void emit(const int sock, const char *buf);
static void active_send_file(Tuser *user, const char *file);
static void passive_send_file(Tuser *user, const char *file);
static void emit_file(const int sock, const char *file, int rest);
static void active_recv_file(Tuser *user, const char *file);
static void passive_recv_file(Tuser *user, const char *file);
static gboolean recv_file(const int sock, const char *file, int rest);



void req_user(Tuser *user, const char *login)
{
	char buf[128];

	if (strlen(login)>64 || !login) {
		update_statusbar("login not between 1 and 64 caracter");
		if (user->rezo->sock>0)
			close(user->rezo->sock);
		desinit_user(user, TRUE);
		return;
	}
	user->rezo->login = strdup(login);
	user->rezo->anonymous = FALSE;
	if (atoi(get_setting(ANONYMOUS_BALISE))==1)
	{
		if ( !strncmp(user->rezo->login, "anonymous", 9)  ||
				 !strncmp(user->rezo->login, "ftp", 3)				||
				 !strncmp(user->rezo->login, "guest", 5)			) 
		{
			user->rezo->anonymous = TRUE;
			ret(user->rezo, RET_331);
			return;
		}
	}
	sprintf(buf, "331 Password required for %s.\r\n", user->rezo->login);
	if (user->rezo->sock>0)
		write(user->rezo->sock, buf, strlen(buf));
}

void req_pass(Tuser *user, const char *pass)
{
	char **users_allowed;
	char *vrai_pass, *pass_cr, *temp;
	char salt[3];
	int i=-1;

	update_statusbar("New user authenticing");
	if (!pass || strlen(pass)>13) {
		bad_lp(user);
		return;
	}
	if (user->rezo->anonymous)
	{
		user->rezo->auth = TRUE;
		post_auth_task(user);
	} else {
		users_allowed = get_settings(USERS_BALISE);
		if (!users_allowed)
			bad_lp(user);
		else	{
			while (users_allowed[++i])
			{
				if ( !strncmp(user->rezo->login, users_allowed[i], strlen(user->rezo->login)) )
				{
					temp = (char *) malloc(strlen(user->rezo->login)+strlen(PASS_PART_BALISE)+1);
					memset(temp, 0, strlen(user->rezo->login)+strlen(PASS_PART_BALISE)+1);
					strncpy(temp, user->rezo->login, strlen(user->rezo->login));
					strncat(temp, PASS_PART_BALISE, strlen(PASS_PART_BALISE));
					vrai_pass = get_setting(temp);
					free(temp);
					salt[0] = *vrai_pass++;
					salt[1] = *vrai_pass--;
					salt[2] = 0;
					pass_cr = crypt(pass, salt);
					if (!strncmp(pass_cr, vrai_pass, strlen(vrai_pass)))
					{
						free(vrai_pass);						/* pass correct */
						user->rezo->auth = TRUE;
						post_auth_task(user);
						break;
					} else {
						free(vrai_pass);
						bad_lp(user);			/* pass incorrect */
						break;
					}
				}
			}
		}
		if (users_allowed && !users_allowed[i])		/* login incorrect */
			bad_lp(user);
		i=-1;									/* un peu de nettoyage */
		if (users_allowed)  {
			while (users_allowed[++i])
				free(users_allowed[i]);
			free(users_allowed);
		}
	}
}

static void bad_lp(Tuser *user)
{
	char *sys_text;
	char bad[]= " entered a bad password, bye!*";

	sys_text = (char *) malloc(strlen(user->name)+strlen(bad)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(bad)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, bad, strlen(bad));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
	ret(user->rezo, RET_532);
	if (user->rezo->sock>0)
		close(user->rezo->sock);
	desinit_user(user, TRUE);
}

static void access_denied(Tuser *user)
{
	char *sys_text;
	char deny[]=" is not allowed to acceed this *";

	sys_text = (char *) malloc(strlen(user->name)+strlen(deny)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(deny)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, deny, strlen(deny));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	user_insert_sys_text(user, sys_text);
	if (sys_text)
		free(sys_text);
	ret(user->rezo, RET_551);
}

static void post_auth_task(Tuser *user)
{
	char sys_txt[]=" is known as ";
	char *sys_text;
	Tuser *last;
	char buf[128];

	/* changement de nom */
	sys_text = (char *) malloc(strlen(user->name)+strlen(user->rezo->login)+strlen(sys_txt)+5);
	memset(sys_text, 0, strlen(user->name)+strlen(user->rezo->login)+strlen(sys_txt)+5);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, sys_txt, strlen(sys_txt));
	if (user->name)
		free(user->name);
	init_user(user->rezo->login, user->rezo);
	last = (Tuser *) (g_slist_nth(list_user, g_slist_length(list_user)-1))->data;
	strncat(sys_text, last->name, strlen(last->name));
	strncat(sys_text, "*", 1);
	desinit_user(user, FALSE);
  user_insert_sys_text((Tuser *) (g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
	
	/* recherche du home */
	if ( find_home_directory(last) )
	{
		if (last->rezo->anonymous)
			ret(last->rezo, RET_230);
		else {
			sprintf(buf, "230 User %s logged in.\r\n", last->rezo->login);
			write(last->rezo->sock, buf, strlen(buf));
		}
	}
}


static gboolean find_home_directory(Tuser *user)
{
	Tuser_path *path;
	char *balise, *rootpath;

	path = (Tuser_path *) malloc(sizeof(Tuser_path));
	memset(path, 0, sizeof(Tuser_path));
	balise = (char *) malloc(strlen(user->rezo->login)+strlen(ROOTDIR_PART_BALISE)+1);
	memset(balise, 0, strlen(user->rezo->login)+strlen(ROOTDIR_PART_BALISE)+1);
	strncpy(balise, user->rezo->login, strlen(user->rezo->login));
	strncat(balise, ROOTDIR_PART_BALISE, strlen(ROOTDIR_PART_BALISE));
	path->root = (char *) malloc(MAXPATHLEN);
	memset(path->root, 0, MAXPATHLEN);
	rootpath = get_setting(balise);
	if ( !rootpath || !realpath(rootpath, path->root) )
	{
		bad_home_directory(user, rootpath);
		free(path);
		return(FALSE);
	}
	if (path->root[strlen(path->root)-1]=='/')
		path->root[strlen(path->root)-1] = 0;
	path->current = (char *) malloc(2);
	memset(path->current, 0, 2);
	strncpy(path->current, "/", 2);
	user->rezo->path = path;
	return(TRUE);
}

static void bad_home_directory(Tuser *user, char *path)
{
	char *sys_text;
	char not_correct[] = " has not a correct home directory ";

	ret(user->rezo, RET_534);
	if (!path)
	{
		path = (char *) malloc(1);
		*path = 0;
	}
	sys_text = (char *) malloc(strlen(user->name)+strlen(not_correct)+strlen(path)+3);
	memset(sys_text, 0, strlen(user->name)+strlen(not_correct)+strlen(path)+3);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, not_correct, strlen(not_correct));
	strncat(sys_text, path, strlen(path));
	strncat(sys_text, "*", 2);
  user_insert_sys_text((Tuser *) (g_slist_nth(list_user, 0))->data, sys_text);
	if (sys_text)
		free(sys_text);
	if (path)
		free(path);
	if (user->rezo->sock>0)
		close(user->rezo->sock);
	desinit_user(user, TRUE);
}


void req_quit(Tuser *user)
{
	char *sys_text;
	char menvais[] = " left, bye!*";

	ret(user->rezo, RET_221);
	sys_text = (char *) malloc(strlen(user->name)+strlen(menvais)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(menvais)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, menvais, strlen(menvais));
  user_insert_sys_text((Tuser *) (g_slist_nth(list_user, 0))->data, sys_text);
	if (sys_text)
		free(sys_text);
	if (user->rezo->sock>0)
		close(user->rezo->sock);
	desinit_user(user, TRUE);
}

void req_type(Tuser *user, const char *type)
{
	char buf[32];

	if (toupper(*type)=='I' || toupper(*type)=='A')
	{
		sprintf(buf, "200 Type set to %c.\r\n", *type);
		write(user->rezo->sock, buf, strlen(buf));
	} else
		ret(user->rezo, RET_501);
}

void req_list(Tuser *user, char *arg)
{
	char *buffer, *path,  **buf, *path_totest, *mask;

	path = arg2path(user, arg);
	path = vire_opt(path);
	path_totest = strdup(path);
	mask = find_mask(path_totest);
	path_totest = is_allowed(user, path_totest, REQ_LIST);
	if (!path_totest)
	{
		access_denied(user);
		return;
	}
	free(path_totest);
	if (!fork())
	{
		user->rezo->thread = pthread_self();
		buf = list(path);
		if (path)
			free(path);
		buffer = toutenun(buf);
/*#ifdef DEBUG
		printf("buf=%s\n", buffer);
#endif*/
		if (user->rezo->pasv) 
		{
			ret(user->rezo, RET_150);
			passive_send(user, buffer);
		} else {
			if (user->rezo->port)
			{	
				ret(user->rezo, RET_150);
				active_send(user, buffer);
			} else {
				free(buffer);
				ret(user->rezo, RET_550);
				exit(1);
			}
		}
		if (buffer)
			free(buffer);
		ret(user->rezo, RET_226);
		exit(1);
	}
	if (path)
		free(path);
}

/*
 * renvoie le path complet (sans / au bout)
 */
static char *arg2path(Tuser *user, const char *arg)
{
	char *ret;
	Tuser_path *path;

	path = user->rezo->path;
	if (!arg)
	{
		ret = malloc(strlen(path->root)+strlen(path->current)+1);
		memset(ret, 0, strlen(path->root)+strlen(path->current)+1);
		strncpy(ret, path->root, strlen(path->root));
		strncat(ret, path->current, strlen(path->current));
	} else {
		ret = malloc(strlen(path->root)+strlen(arg)+strlen(path->current)+2);
		memset(ret, 0, strlen(path->root)+strlen(arg)+strlen(path->current)+2);
		strncpy(ret, path->root, strlen(path->root));
		if (arg[0]!='/')
			strncat(ret, path->current, strlen(path->current));
		if (ret[strlen(ret)-1]=='/' && arg[0]=='/')
			strncat(ret, arg+1, strlen(arg+1));
		else {
			if (ret[strlen(ret)-1]!='/' && arg[0]!='/')
			{
				strncat(ret, "/", 1);
				strncat(ret, arg, strlen(arg));
			} else
				strncat(ret, arg, strlen(arg));
		}
	}
	if (ret[strlen(ret)-1]=='/')
		ret[strlen(ret)-1]=0;
	return(ret);
}


static char *toutenun(char **buffer)
{
	char *ret;
	int i=-1, size=1;

	if (!buffer || !buffer[0])	return(NULL);
	while(buffer[++i])
		size += strlen(buffer[i])+2;
	ret = (char *) malloc(size);
	memset(ret, 0, size);
	strncpy(ret, buffer[0], strlen(buffer[0]));
	strncat(ret, "\r\n", 2);
	while (--i>0)
	{
		strncat(ret, buffer[i], strlen(buffer[i]));
		strncat(ret, "\r\n", 2);
	}
	/* on free buffer */
	i=-1;
	while (buffer[++i])
		free(buffer[i]);
	if (buffer)
		free(buffer);
	return(ret);
}

static void active_send(const Tuser *user, char *buffer)
{
	struct sockaddr_in addr;
	unsigned long int tmp_addr;
	char ip[16];

	if ( (user->rezo->port->sock=socket(AF_INET, SOCK_STREAM, 0))==-1 )
	{
		ret(user->rezo, RET_501);
		return;
	}
	memset(ip, 0, 16);
	sprintf(ip, "%d.%d.%d.%d", user->rezo->port->ip[0], user->rezo->port->ip[1],
			user->rezo->port->ip[2], user->rezo->port->ip[3]);
	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(user->rezo->port->port);
	tmp_addr = inet_addr(ip);
	memcpy(&addr.sin_addr, &tmp_addr, sizeof(tmp_addr));
	if ( connect(user->rezo->port->sock, (struct sockaddr *) &addr, sizeof(struct sockaddr)) )
	{
		ret(user->rezo, RET_501);
		perror("connect");
		return;
	}
	if (buffer)
		emit(user->rezo->port->sock, buffer);
	if (user->rezo->port->sock>0)
		close(user->rezo->port->sock);
}


static void passive_send(const Tuser *user, char *buffer)
{
	int sock, size;

	size = sizeof(struct sockaddr_in);
	sock = accept(user->rezo->pasv->sock, &user->rezo->pasv->addr, &size);
	if (sock<0)
	{
		ret(user->rezo, RET_501);
		return;
	}
	if (buffer)
		emit(sock, buffer);
	if (sock>0)
		close(sock);
}

static void emit(const int sock, const char *buf)
{
	struct timeval time;
	fd_set fds;
	int n=-1;
	gboolean all_sent = FALSE;

	while (!all_sent)
	{
		FD_ZERO(&fds);
		FD_SET(sock, &fds);
		time.tv_sec = 0;
		time.tv_usec = 0;
		n = select(sock+1, NULL, &fds, NULL, &time);
		if (n>0)
		{
			if ( strlen(buf) > OUT_BUF )
			{
				write(sock, buf, OUT_BUF);
				buf += OUT_BUF;
			} else {
				write(sock, buf, strlen(buf));
				all_sent = TRUE;
			}
		}
	}
}


void req_port(Tuser *user, const char *arg)
{
	int porth, portl;
	Tuser_port *port;

	port = (Tuser_port *) malloc(sizeof(Tuser_port));
	memset(port, 0, sizeof(Tuser_port));
	if ( sscanf(arg, "%d,%d,%d,%d,%d,%d", &port->ip[0], &port->ip[1],
					&port->ip[2], &port->ip[3], &porth, &portl) < 6 )
	{
		ret(user->rezo, RET_502);
		if (port)
			free(port);
	} else {
		port->port = ((porth&0xFF)<<8)|(portl&0xFF);
		if (port->port<=0 || port->port>65536)
			ret(user->rezo, RET_502);
		else {
			user->rezo->port = port;
			ret(user->rezo, RET_200);
			if (user->rezo->pasv)
				free(user->rezo->pasv);
		}
	}
}




void req_cwd(Tuser *user, char *arg)
{
	char *path;
	struct stat st;
	Tuser_path *u_path;

	path = arg2path(user, arg);
	path = is_allowed(user, path, REQ_CWD);
	if (!path)
	{
		free(path);
		access_denied(user);
		return;
	}
	if (!stat(path,  &st) && (st.st_mode & S_IFDIR) )
	{
		u_path = malloc(sizeof(Tuser_path));
		memset(u_path, 0, sizeof(Tuser_path));
		u_path->current = malloc(strlen(path)-strlen(user->rezo->path->root)+2);
		memset(u_path->current, 0, strlen(path)-strlen(user->rezo->path->root)+2);
		strncpy(u_path->current, path+strlen(user->rezo->path->root),
				strlen(path)-strlen(user->rezo->path->root));
		strncat(u_path->current, "/", 1);
		u_path->root = user->rezo->path->root;
		if (user->rezo && user->rezo->path)
			free(user->rezo->path);
		user->rezo->path = u_path;
		ret(user->rezo, RET_250);
	} else
		access_denied(user);
	if (path);
		free(path);
}

void req_pwd(Tuser *user)
{
	char *buf;
	char avt[]="257 \"";
	char apr[]="\" is current directory.\r\n";

	buf = (char *) malloc(strlen(avt)+strlen(apr)+strlen(user->rezo->path->current)+1);
	memset(buf, 0, strlen(avt)+strlen(apr)+strlen(user->rezo->path->current)+1);
	strncpy(buf, avt, strlen(avt));
	strncat(buf, user->rezo->path->current, strlen(user->rezo->path->current));
	strncat(buf, apr, strlen(apr));
	write(user->rezo->sock, buf, strlen(buf));
}


void req_size(Tuser *user, char *arg)
{
	char *path=NULL;
	struct stat st;
	char size[16];

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path = is_allowed(user, path, REQ_SIZE);
	if (!path)
	{
		free(path);
		access_denied(user);
		return;
	}
	if (!stat(path, &st) )
	{
		if ( S_ISREG(st.st_mode) )
		{
			memset(size, 0, 16);
			sprintf(size, "213 %ld\r\n", (long int)st.st_size);
			write(user->rezo->sock, size, strlen(size));
		} else
			ret(user->rezo, RET_552);
	} else
		access_denied(user);
	if (path)
		free(path);
}


void req_mdtm(Tuser *user, char *arg)
{
	char *path=NULL;
	struct stat st;
	char time[16], retu[32];

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path = is_allowed(user, path, REQ_MDTM);
	if (!path)
	{
		free(path);
		access_denied(user);
		return;
	}
	if (!stat(path, &st) )
	{
		if ( S_ISREG(st.st_mode) )
		{
			memset(time, 0, 16);
			memset(retu, 0, 32);
			strftime(time, sizeof(time), "%Y%m%d%H%M%S", gmtime(&st.st_mtime));
			sprintf(retu, "213 %s\r\n", time);
			write(user->rezo->sock, retu, strlen(retu));
		} else
			ret(user->rezo, RET_552);
	} else
		access_denied(user);
	if (path)
		free(path);
}


void req_pasv(Tuser *user)
{
	Tuser_pasv *pasv;
	int port;
	unsigned char *ip;
	char buf[128];

	if (user->rezo->pasv && user->rezo->pasv->sock>0)
	{
		close(user->rezo->pasv->sock);
		user->rezo->pasv->sock = 0;
	}
	pasv = (Tuser_pasv *) malloc(sizeof(Tuser_pasv));
	memset(pasv, 0, sizeof(Tuser_pasv));
	if ( (pasv->sock=socket(AF_INET, SOCK_STREAM, 0))<0 )
	{
		free(pasv);
		ret(user->rezo, RET_425);
	} else {
		for (port=1025;port<65536;port++)
		{
			memset(&pasv->addr, 0, sizeof(struct sockaddr_in));
			pasv->addr.sin_family = AF_INET;
			pasv->addr.sin_port = htons(port);
			pasv->addr.sin_addr.s_addr = htonl(INADDR_ANY);
			if (!bind(pasv->sock, (struct sockaddr *) &pasv->addr, sizeof(struct sockaddr)) )
				break;
		}
		if (port>=65536)
		{
			close(pasv->sock);
			free(pasv);
			ret(user->rezo, RET_425);
		} else {
			if ( listen(pasv->sock, 1)<0 )
			{
				close(pasv->sock);
				free(pasv);
				ret(user->rezo, RET_425);
			} else {
				ip = getlocalip(user->rezo->sock);
				sprintf(buf,  "227 Entering Passive Mode (%d,%d,%d,%d,%d,%d)\r\n", ip[0], ip[1], ip[2], ip[3],
							((port>>8)&0xFF), (port&0xFF) );
				write(user->rezo->sock, buf, strlen(buf));
				user->rezo->pasv = pasv;
				free(user->rezo->port);
			}
		}
	}
}

/* ip doit avoir 4 octet d'alloue */
static unsigned char *getlocalip(int sock)
{
	int size;
	struct sockaddr_in addr;
	static unsigned char ip[4];

	size = sizeof(struct sockaddr);
	memset(ip, 0, sizeof(ip));
	if ( !getsockname(sock, (struct sockaddr *) &addr, &size) )
		memcpy(ip, &addr.sin_addr, sizeof(ip));
	return(ip);
}

void req_mkd(Tuser *user, char *arg)
{
	char *path = NULL;
	char *path_totest = NULL;
	char *new_dir=NULL;
	int i;

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path_totest = strdup(path);
	for (i=strlen(path_totest)-1;i>=0;i--)
	{
		if (path_totest[i]=='/')
		{
			path_totest[i] = 0;
			new_dir = strdup(path_totest+i+1);
			break;
		}
	}
	path_totest = is_allowed(user, path_totest, REQ_MKD);
	if (!path_totest)
		access_denied(user);
	else {	
		path_totest = realloc(path_totest, strlen(path_totest)+strlen(new_dir)+2);
		memset(path_totest+strlen(path_totest), 0, strlen(new_dir)+2);
		strncat(path_totest, "/", 1);
		strncat(path_totest, new_dir, strlen(new_dir));
	
		if ( mkdir(path_totest, UMASK_MKD) )
			ret(user->rezo, RET_553);
		else
			ret(user->rezo, RET_257);
	}
	if (path)
		free(path);
	if (path_totest)
		free(path_totest);
	if (new_dir)
		free(new_dir);
}


void req_rmd(Tuser *user, char *arg)
{
	char *path=NULL;
	char *path_totest=NULL;
	char *new_dir=NULL;
	int i;

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path_totest = strdup(path);
	for (i=strlen(path_totest)-1;i>=0;i--)
	{
		if (path_totest[i]=='/')
		{
			path_totest[i] = 0;
			new_dir = strdup(path_totest+i+1);
			break;
		}
	}
	path_totest = is_allowed(user, path_totest, REQ_RMD);
	if (!path_totest)
		access_denied(user);
	else {
		path_totest = realloc(path_totest, strlen(path_totest)+strlen(new_dir)+2);
		memset(path_totest+strlen(path_totest), 0, strlen(new_dir)+2);
		strncat(path_totest, "/", 1);
		strncat(path_totest, new_dir, strlen(new_dir));
	
		if (!rmdir(path_totest))
			ret(user->rezo, RET_251);
		else
			access_denied(user);
	}
	if (path)
		free(path);
	if (path_totest)
		free(path_totest);
	if (new_dir)
		free(new_dir);
}


void req_dele(Tuser *user, char *arg)
{
	char *path=NULL;
	char *path_totest=NULL;
	char *new_dir=NULL;
	int i;

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path_totest = strdup(path);
	for (i=strlen(path_totest)-1;i>=0;i--)
	{
		if (path_totest[i]=='/')
		{
			path_totest[i] = 0;
			new_dir = strdup(path_totest+i+1);
			break;
		}
	}
	path_totest = is_allowed(user, path_totest, REQ_DELE);
	if (!path_totest)
		access_denied(user);
	else {
		path_totest = realloc(path_totest, strlen(path_totest)+strlen(new_dir)+2);
		memset(path_totest+strlen(path_totest), 0, strlen(new_dir)+2);
		strncat(path_totest, "/", 1);
		strncat(path_totest, new_dir, strlen(new_dir));
	
		if (!unlink(path_totest))
			ret(user->rezo, RET_251);
		else
			access_denied(user);
	}
	if (path)
		free(path);
	if (path_totest)
		free(path_totest);
	if (new_dir)
		free(new_dir);
}


void req_nlst(Tuser *user, char *arg)
{
	req_list(user, arg);
}


void req_cdup(Tuser *user)
{
	char up[]="..";

	req_cwd(user, up);
}



void req_rest(Tuser *user, char *arg)
{
	user->rezo->rest = atoi(arg);
	ret(user->rezo, RET_350);
}


void req_abor(Tuser *user)
{
	pthread_kill(user->rezo->thread, SIGINT);
	if (user->rezo->pasv && user->rezo->pasv->sock>0)
		close(user->rezo->pasv->sock);
	if (user->rezo->port && user->rezo->port->sock>0)
		close(user->rezo->port->sock);
	ret(user->rezo, RET_227);
}


void req_help(Tuser *user, char *arg)
{
	ret(user->rezo, RET_214);
}


void req_retr(Tuser *user, char *arg)
{
	char *path=NULL, *path_totest=NULL, *new_dir=NULL;
	int i, len;

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path_totest = strdup(path);
	for (i=strlen(path_totest)-1;i>=0;i--)
	{
		if (path_totest[i]=='/')
		{
			path_totest[i] = 0;
			new_dir = strdup(path_totest+i+1);
			break;
		}
	}
	path_totest = is_allowed(user, path_totest, REQ_RETR);
	if (!path_totest)
		access_denied(user);
	else {
		len = strlen(path_totest);
		path_totest = realloc(path_totest, len+strlen(new_dir)+2);
		memset(path_totest+strlen(path_totest), 0, strlen(new_dir)+2);
		strncat(path_totest, "/", 1);
		strncat(path_totest, new_dir, strlen(new_dir));
		if (!fork())
		{
			user->rezo->thread = pthread_self();
			if (user->rezo->pasv) 
			{
				ret(user->rezo, RET_151);
				passive_send_file(user, path_totest);
			} else {
				if (user->rezo->port)
				{	
					ret(user->rezo, RET_151);
					active_send_file(user, path_totest);
				} else {
					ret(user->rezo, RET_550);
					exit(1);
				}
			}
			ret(user->rezo, RET_226);
			exit(1);
		}
	}
	if (path)
		free(path);
	if (path_totest)
		free(path_totest);
	if (new_dir)
		free(new_dir);
}


static void passive_send_file(Tuser *user, const char *file)
{
	int sock, size;

	size = sizeof(struct sockaddr_in);
	sock = accept(user->rezo->pasv->sock, &user->rezo->pasv->addr, &size);
	if (sock<0)
	{
		ret(user->rezo, RET_501);
		return;
	}
	if (file)
		emit_file(sock, file, user->rezo->rest);
	user->rezo->rest = 0;
	close(sock);
}


static void active_send_file(Tuser *user, const char *file)
{
	struct sockaddr_in addr;
	unsigned long int tmp_addr;
	char ip[16];

	if ( (user->rezo->port->sock=socket(AF_INET, SOCK_STREAM, 0))==-1 )
	{
		ret(user->rezo, RET_501);
		return;
	}
	memset(ip, 0, 16);
	sprintf(ip, "%d.%d.%d.%d", user->rezo->port->ip[0], user->rezo->port->ip[1],
			user->rezo->port->ip[2], user->rezo->port->ip[3]);
	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(user->rezo->port->port);
	tmp_addr = inet_addr(ip);
	memcpy(&addr.sin_addr, &tmp_addr, sizeof(tmp_addr));
	if ( connect(user->rezo->port->sock, (struct sockaddr *) &addr, sizeof(struct sockaddr)) )
	{
		ret(user->rezo, RET_501);
		perror("connect");
		return;
	}
	if (file)
		emit_file(user->rezo->port->sock, file, user->rezo->rest);
	user->rezo->rest = 0;
	close(user->rezo->port->sock);
}

static void emit_file(const int sock, const char *file, int rest)
{
	struct timeval time;
	fd_set fds;
	int n=-1, size_tot, m;
	gboolean all_sent = FALSE;
	FILE *fd;
	char buf[BUF_FILE];

	if ( !(fd=fopen(file, "r")) )
		return;
	fseek(fd, 0, SEEK_END);
	size_tot = ftell(fd);
	fseek(fd, rest, SEEK_SET);
	while (!all_sent)
	{
		FD_ZERO(&fds);
		FD_SET(sock, &fds);
		time.tv_sec = 0;
		time.tv_usec = 0;
		n = select(sock+1, NULL, &fds, NULL, &time);
		if (n>0)
		{
			memset(buf, 0, BUF_FILE);
			m = fread(buf, 1, BUF_FILE, fd);
			if ( m<0 )
				return;
			if (m==sizeof(buf))
			{
				write(sock, buf, BUF_FILE);
			} else {
				if (m>0)
					write(sock, buf, strlen(buf));
				all_sent = TRUE;
			}
		}
	}
	fclose(fd);
}


void req_stor(Tuser *user, char *arg)
{
	char *path, *path_totest, *new_dir=NULL;
	int i;

	if (!arg)
	{
		ret(user->rezo, RET_501);
		return;
	}
	path = arg2path(user, arg);
	path_totest = strdup(path);
	for (i=strlen(path_totest)-1;i>=0;i--)
	{
		if (path_totest[i]=='/')
		{
			path_totest[i] = 0;
			new_dir = strdup(path_totest+i+1);
			break;
		}
	}
	path_totest = is_allowed(user, path_totest, REQ_STOR);
	if (!path_totest)
		access_denied(user);
	else {
		path_totest = realloc(path_totest, strlen(path_totest)+strlen(new_dir)+2);
		memset(path_totest+strlen(path_totest), 0, strlen(new_dir)+2);
		strncat(path_totest, "/", 1);
		strncat(path_totest, new_dir, strlen(new_dir));
		if (!fork())
		{
			user->rezo->thread = pthread_self();
			if (user->rezo->pasv) 
			{
				ret(user->rezo, RET_151);
				passive_recv_file(user, path_totest);
			} else {
				if (user->rezo->port)
				{	
					ret(user->rezo, RET_151);
					active_recv_file(user, path_totest);
				} else {
					TRACE;
					ret(user->rezo, RET_550);
					exit(1);
				}
			}
			ret(user->rezo, RET_226);
			exit(1);
		}
	}
	free(path);
	free(path_totest);
	free(new_dir);
}


static void passive_recv_file(Tuser *user, const char *file)
{
	int sock, size;

	size = sizeof(struct sockaddr_in);
	sock = accept(user->rezo->pasv->sock, &user->rezo->pasv->addr, &size);
	if (sock<0)
	{
		ret(user->rezo, RET_501);
		return;
	}
	if (file)
		recv_file(sock, file, user->rezo->rest);
	user->rezo->rest = 0;
	close(sock);
}


static void active_recv_file(Tuser *user, const char *file)
{
	struct sockaddr_in addr;
	unsigned long int tmp_addr;
	char ip[16];

	if ( (user->rezo->port->sock=socket(AF_INET, SOCK_STREAM, 0))==-1 )
	{
		ret(user->rezo, RET_501);
		return;
	}
	memset(ip, 0, 16);
	sprintf(ip, "%d.%d.%d.%d", user->rezo->port->ip[0], user->rezo->port->ip[1],
			user->rezo->port->ip[2], user->rezo->port->ip[3]);
	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(user->rezo->port->port);
	tmp_addr = inet_addr(ip);
	memcpy(&addr.sin_addr, &tmp_addr, sizeof(tmp_addr));
	if ( connect(user->rezo->port->sock, (struct sockaddr *) &addr, sizeof(struct sockaddr)) )
	{
		ret(user->rezo, RET_501);
		perror("connect");
		return;
	}
	if (file)
		if ( !recv_file(user->rezo->port->sock, file, user->rezo->rest) )
			access_denied(user);
	user->rezo->rest = 0;
	close(user->rezo->port->sock);
}


static gboolean recv_file(const int sock, const char *file, int rest)
{
	struct timeval time;
	fd_set fds;
	int n=-1, m;
	gboolean all_recv = FALSE;
	FILE *fd;
	char buf[BUF_FILE];

	if ( (fd=fopen(file, "r")) && !atoi(get_setting(OVERWRITE_BALISE)) )
		return(FALSE);
	if (fd)
		fclose(fd);

	if ( !(fd=fopen(file, "a")) )
		return(FALSE);
	fseek(fd, rest, SEEK_SET);
	while (!all_recv)
	{
		FD_ZERO(&fds);
		FD_SET(sock, &fds);
		time.tv_sec = 0;
		time.tv_usec = 0;
		n = select(sock+1, &fds, NULL, NULL, &time);
		if (n>0)
		{
			memset(buf, 0, BUF_FILE);
			m = read(sock, buf, BUF_FILE);
			if ( m<0 )
				return(TRUE);
			if (m==BUF_FILE)
			{
				fwrite(buf, 1, BUF_FILE, fd);
			} else {
				if (m>0)
					fwrite(buf, 1, m, fd);
				all_recv = TRUE;
			}
		}
	}
	fclose(fd);
	return(TRUE);
}
