#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <ctype.h>
#include "statusbar.h"
#include "conf.h"
#include "list.h"
#include "debug.h"
#include "s_ret.h"
#include "s_req.h"
#include "s_ret_str.h"
#include "server.h"
#include "text.h"
#include "secu.h"


struct server {
	int sock;
	struct sockaddr_in addr;
};


static gboolean check_conn();
static gboolean check_data();
static gboolean check_timeout();
static void get_mesg(Tuser *user);
static void reaction_mesg(Tuser *user, int req, char *arg);
static void user_left(Tuser *user);
static void bad_command(Tuser *user);
static void not_allowed(Tuser *user);
static void clean_buf(char *buf);

struct server serv;


gboolean init_server()
{
	int i=1, size, addr;

	update_statusbar("Starting server...");
	serv.sock = socket(AF_INET, SOCK_STREAM, 0);
	if (serv.sock==-1)  {
		perror("Creation du socket serveur");
		return(FALSE);
	}
	setsockopt(serv.sock, SOL_SOCKET, SO_REUSEADDR, (void *)&i, sizeof(i));
	size = sizeof(struct sockaddr_in);
	memset(&serv.addr, 0, size);
	serv.addr.sin_family = PF_INET;
	if (!get_setting(PORT_BALISE))
		return(FALSE);
	serv.addr.sin_port = htons(atoi(get_setting(PORT_BALISE)));
	addr = INADDR_ANY;
	memcpy(&serv.addr.sin_addr.s_addr, &addr, sizeof(int));
	if (bind(serv.sock, (struct sockaddr *)&serv.addr, size)<0)  {
		perror("Error: Is there anothere FTP server ?");
		return(FALSE);
	}
	if (listen(serv.sock, atoi(get_setting(MAXUSER_BALISE))+2)<0)  {
		perror("Error: can't listen");		// on renvoie un message d'erreur plus loin
		return(FALSE);
	}
	return(TRUE);
}

gboolean update_server()
{
	if (!check_conn() || !check_data() || !check_timeout())
		return(FALSE);
	return(TRUE);
}


void desinit_server()
{
	if (serv.sock>0)
		close(serv.sock);
}


void reinit_server()
{
	desinit_server();
	init_server();
}


static gboolean check_conn()
{
	int n, size, sock;
	fd_set fds;
	struct timeval time;
	struct sockaddr_in addr;
	Tuser_nt *nt;

	size = sizeof(struct sockaddr_in);
	FD_ZERO(&fds);
	FD_SET(serv.sock, &fds);
	if (!get_setting(TIMEOUT_SELECT_BALISE))
		return(FALSE);
	time.tv_sec = atoi(get_setting(TIMEOUT_SELECT_BALISE));
	time.tv_usec = 10;
	n = select(serv.sock+1, &fds, NULL, NULL, &time);
	if (n>0)
	{
		sock = accept(serv.sock, (struct sockaddr *) &addr, &size);
		nt = (Tuser_nt *) malloc(sizeof(Tuser_nt));
		memset(nt, 0, sizeof(Tuser_nt));
		nt->sock = sock;
		nt->addr = &addr;
		nt->auth = FALSE;
		nt->login = NULL;
		nt->port = NULL;
		nt->pasv = NULL;
		nt->rest = 0;
		memset(nt->ip, 0, 4);
		gettimeofday(&nt->lastop, NULL);
	  if ( g_slist_length(list_user)>=atoi(get_setting(MAXUSER_BALISE)) )  {
			update_statusbar("Refusing new connexion : too many users");
			ret(nt, RET_530);
			close(nt->sock);
			free(nt);
			return(TRUE);
		}
		if ( is_banned(&nt->addr->sin_addr) )	{
			user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, "* Refusing new connexion: banned");
			update_statusbar("Refusing new connexion : banned");
			ret(nt, RET_535);
			close(nt->sock);
			free(nt);
			return(TRUE);
		}
		update_statusbar("Accepting new connexion");
		init_user(NULL, nt);
		ret(nt, RET_220);
	}
	return(TRUE);
}

static gboolean check_data()
{
	fd_set in, ex;
	int size, i, n;
	struct timeval time;
	Tuser *user;

	size = sizeof(struct sockaddr_in);
	time.tv_sec = atoi(get_setting(TIMEOUT_SELECT_BALISE));
	time.tv_usec = 0;
	for (i=g_slist_length(list_user)-1;i>0;i--)  /* on ne prends pas en compte le user All */
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user->rezo && user->rezo->sock>0)
		{
			FD_ZERO(&in);
			FD_ZERO(&ex);
			FD_SET(user->rezo->sock, &in);
			FD_SET(user->rezo->sock, &ex);
			n = select(user->rezo->sock+1, &in, NULL, &ex, &time);
			if (n>0)
			{
				if (FD_ISSET(user->rezo->sock, &in))
				{
					get_mesg(user);
					break;
				} else {
					if (FD_ISSET(user->rezo->sock, &ex))
					{
						user_insert_sys_text(user, "* User left *");
						if (user->rezo->sock>0)
							close(user->rezo->sock);
				  	desinit_user(user, TRUE);
					}
				}
			}
		}
	}
	return(TRUE);
}

static gboolean check_timeout()
{
	struct timeval time;
	int i;
	Tuser *user;
	char sys_text[128];

	gettimeofday(&time, NULL);
	if (!get_setting(TIMEOUT_USER_BALISE))
		return(FALSE);
	for (i=g_slist_length(list_user)-1;i>0;i--)   /* on ne prend pas en compte l'utilisateur All */
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if ( user->rezo && (time.tv_sec - user->rezo->lastop.tv_sec) > atoi(get_setting(TIMEOUT_USER_BALISE)) )
		{
			memset(sys_text, 0, 128);
			strncpy(sys_text, "*", 1);
			strncat(sys_text, user->name, strlen(user->name));
			strncat(sys_text, " has exceeded the timeout, bye !*", 32);
			user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
			ret(user->rezo, RET_421);
			if (user->rezo->sock>0)
		    close(user->rezo->sock);
			desinit_user(user, TRUE);
		}
	}
	return(TRUE);
}

void ret(Tuser_nt *nt, int mesg)
{
	write(nt->sock, ftp_ret_str[mesg], strlen(ftp_ret_str[mesg]));
}


static void get_mesg(Tuser *user)
{
	char buf[1024];
	char *arg;
	int size, i;
	gboolean goodcmd;

	size = recv(user->rezo->sock, buf, sizeof(buf), 0);
	if ( size==1024)
	{
		update_statusbar("User enters a command too long => deconnected");
		ret(user->rezo, RET_500);
		if (user->rezo->sock>0)
			close(user->rezo->sock);
		desinit_user(user, TRUE);
		return;
	}
	if ( !size ) {
		user_left(user);
		return;
	}
	gettimeofday(&user->rezo->lastop, NULL);
	buf[size-2] = '\0';
	clean_buf(buf);
	i=-1;
	while (buf[++i] && buf[i]!=' ')
		buf[i] = toupper(buf[i]);
	if (!buf[i])
		arg=NULL;
	else
		arg = buf+i+1;
  user_insert_text(user, buf, SMALL_TAG);
	i=-1; 
	goodcmd=FALSE;
	while (request[++i])
	{
		if ( !strncmp(buf, request[i], strlen(request[i])) &&
				(buf[strlen(request[i])]==' ' || buf[strlen(request[i])]==0) ) {
			goodcmd = TRUE;
			break;
		}
	}
	if (!goodcmd)
		i=-1;
	reaction_mesg(user, i, arg);
}

static void clean_buf(char *buf)
{
	while (buf[strlen(buf)-1]=='\n' || buf[strlen(buf)-1]=='\r' || buf[strlen(buf)-1]==' ')
		buf[strlen(buf)-1]=0;
}


static void user_left(Tuser *user)
{
	char *sys_text;
	char bye[] = " has left, bye!*";

	sys_text = (char *) malloc(strlen(user->name)+strlen(bye)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(bye)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, bye, strlen(bye));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
	if (user->rezo->sock>0)
		close(user->rezo->sock);
	desinit_user(user, TRUE);
}


static void bad_command(Tuser *user)
{
	char *sys_text;
	char badcmd[] = " has entered a bad command*";

	ret(user->rezo, RET_500);
	sys_text = (char *) malloc(strlen(user->name)+strlen(badcmd)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(badcmd)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, badcmd, strlen(badcmd));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
}

static void not_allowed(Tuser *user)
{
	char *sys_text;
	char notallowed[]=" has entered a command while not logged*";

	ret(user->rezo, RET_531);
	sys_text = (char *) malloc(strlen(user->name)+strlen(notallowed)+2);
	memset(sys_text, 0, strlen(user->name)+strlen(notallowed)+2);
	strncpy(sys_text, "*", 1);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, notallowed, strlen(notallowed));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
}


static void reaction_mesg(Tuser *user, int req, char *arg)
{
	if (!user->rezo->auth)
	{
		switch (req)	{
			case REQ_USER:	req_user(user, arg);	break;
			case REQ_PASS:	req_pass(user, arg);	break;
			case REQ_QUIT:	req_quit(user);	break;
			case REQ_SYST:	
			case REQ_TYPE:
			case REQ_LIST:
			case REQ_PORT:
			case REQ_NLST:
			case REQ_CDUP:	
		  case REQ_CWD:
			case REQ_PWD:	
			case REQ_RETR:
			case REQ_REST:
			case REQ_SIZE:
			case REQ_MDTM:
			case REQ_STOR:
			case REQ_NOOP:
			case REQ_PASV:
			case REQ_ABOR:
			case REQ_MKD:
			case REQ_RMD:
			case REQ_HELP:
			case REQ_DELE:		not_allowed(user);	break;
			default:			bad_command(user);
		}
	} else {
		switch (req)	{
			case REQ_SYST:	ret(user->rezo, RET_215);	break;
			case REQ_QUIT:	req_quit(user);	break;
			case REQ_TYPE:	req_type(user, arg); break;
			case REQ_LIST:	req_list(user, arg);	break;
			case REQ_PORT:	req_port(user, arg);	break;
			case REQ_NLST:	req_nlst(user, arg);	break;
			case REQ_CDUP:	req_cdup(user);				break;
			case REQ_CWD:		req_cwd(user, arg);		break;
			case REQ_PWD:		req_pwd(user);				break;
			case REQ_RETR:	req_retr(user, arg);	break;
			case REQ_REST:	req_rest(user, arg);	break;
			case REQ_SIZE:	req_size(user, arg);	break;
			case REQ_MDTM:	req_mdtm(user, arg);	break;
			case REQ_STOR:	req_stor(user, arg);	break;
			case REQ_NOOP:	ret(user->rezo, RET_201);	break;
			case REQ_PASV:	req_pasv(user);				break;
			case REQ_ABOR:	req_abor(user, arg);	break;
			case REQ_MKD:		req_mkd(user, arg);		break;
			case REQ_RMD:		req_rmd(user, arg);		break;
			case REQ_HELP:	req_help(user, arg);	break;
			case REQ_DELE:	req_dele(user, arg);	break;
			default:		bad_command(user);
		}
	}
}


void kick_user(Tuser *user)
{
	char kicked[]=" has been kicked !";
	char *sys_text;

	sys_text = (char *) malloc(strlen(user->name)+strlen(kicked)+3);
	memset(sys_text, 0, strlen(user->name)+strlen(kicked)+3);
	strncpy(sys_text, "* ", 2);
	strncat(sys_text, user->name, strlen(user->name));
	strncat(sys_text, kicked, strlen(kicked));
	user_insert_sys_text((Tuser *)(g_slist_nth(list_user, 0))->data, sys_text);
	free(sys_text);
	if (user->rezo->sock>0)
	{
		ret(user->rezo, RET_536);
		close(user->rezo->sock);
	}
	desinit_user(user, TRUE);
}
