#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>
#include "sys_cmd.h"
#include "debug.h"

#define BUF_SIZE 256

static int match(char *name, char *mask);
static char *perm_date_size(char *file, char *path);

char **list(char *arg)
{
	char *path, *mask=NULL, *perm_date_siz;
	char **ret;
	char buf[BUF_SIZE];
	DIR *dir;
	struct dirent *entr;
	int cptr=0;

	if (!arg)
	{
		path = (char *) malloc(2);
		memset(path, 0, 2);
		*path = '/';  /* current dir */
	} else {
		/* getrealdir */
		path = vire_opt(arg);
		mask = find_mask(path);
		if (strlen(path)==0)
		{
			path = (char *) malloc(2);
			memset(path, 0, 2);
			*path='/';
		}
	}

	if ( !(dir=opendir(path)) )
		return(NULL);
	ret = (char **) malloc(sizeof(ret));
	memset(ret, 0, sizeof(ret));
	while ( (entr=readdir(dir)) )
	{
		if (entr->d_name[0]=='.')
			continue;
		if (match(entr->d_name, mask))
		{
			ret = (char **) realloc(ret, sizeof(ret)*++cptr+sizeof(ret));
			//memset(ret+sizeof(ret)*cptr, 0, sizeof(ret));
			perm_date_siz = perm_date_size(entr->d_name, path);
			memset(buf, 0, BUF_SIZE);
			sprintf(buf, "%s\t%s", perm_date_siz, entr->d_name);
			free(perm_date_siz);
			ret[cptr-1] = (char *) malloc(strlen(buf)+1);
			memset(ret[cptr-1], 0, strlen(buf)+1);
			strncpy(ret[cptr-1], buf, strlen(buf));
		}
	}
	ret[cptr]=NULL;
	closedir(dir);
	return(ret);
}


static char *perm_date_size(char *file, char *path)
{
	char *fp;
	char *ret;
	char date[128];
	struct stat st;
	time_t time_sec;
	struct tm *time_tm;
	int year;

	ret = malloc(128);
	memset(ret, 0, 128);
	fp = malloc(strlen(file)+strlen(path)+2);
	memset(fp, 0, strlen(file)+strlen(path)+2);
	strncpy(fp, path, strlen(path));
	strncat(fp, "/", 1);
	strncat(fp, file, strlen(file));
	if ( stat(fp, &st)>=0)
	{
		time_sec = time(NULL);
		time_tm = localtime(&time_sec);
		year = time_tm->tm_year;
		time_tm = localtime(&st.st_mtime);
		if (time_tm->tm_year==year)
			strftime(date, sizeof(date), "%b %d %H:%M", time_tm);
		else
			strftime(date, sizeof(date), "%b %d  %Y", time_tm);

		sprintf(ret, "%c%c%c%c%c%c%c%c%c%c %3d ftp ftp %13ld %s", 
			S_ISDIR(st.st_mode) ? 'd' : S_ISLNK(st.st_mode) ? 'l' : '-',
			st.st_mode & S_IRUSR ? 'r' : '-',
			st.st_mode & S_IWUSR ? 'w' : '-',
			st.st_mode & S_IXUSR ? 'x' : '-',
			st.st_mode & S_IRGRP ? 'r' : '-',
			st.st_mode & S_IWGRP ? 'w' : '-',
			st.st_mode & S_IXGRP ? 'x' : '-',
			st.st_mode & S_IROTH ? 'r' : '-',
			st.st_mode & S_IWOTH ? 'w' : '-',
			st.st_mode & S_IXOTH ? 'x' : '-',
			(int)st.st_nlink, st.st_size, date);
	}
	free(fp);
	return(ret);
}


static int match(char *name, char *mask)
{
	int i;

	if (!mask) return(1);
	for (i=strlen(mask)-1;i>=0;i--)
		if (mask[i]=='*')
			break;
	if (i<strlen(mask)-1)
		if (strncmp(mask+i+1, name+strlen(name)-(strlen(mask)-i-1), strlen(mask)-i-1))
			return(0);
	if (i>0)
		if (strncmp(mask, name, i))
			return(0);
	return(1);
}


char *find_mask(char *path)
{
	int i;
	int star=0;
	char *mask=NULL;

	i=strlen(path);
	while (path[--i]!='/')
		if (path[i]=='*')
			star++;
	if (star)
	{
		path[i]=0;
		mask = path+i+1;
	}
	return(mask);
}

/* on s'en fout, on renvoie toujours le ls -la */
char *vire_opt(unsigned char *arg)
{
	unsigned int i;
	char *path=NULL;
	int debut=-1, fin=-1;

	for (i=0;i<strlen(arg)-1;i++)
	{
		if ( debut>=0 && (arg[i]==' ' || (arg[i]=='-' && (i>0 && arg[i-1]=='/') ) ) )
		{
			fin=i;
			break;
		}
		if (arg[i]=='/' && debut<0)
			debut=i;
	}
	if (fin<0)
		fin=strlen(arg)-1;
	path = (char *) malloc(fin-debut+2);
	memset(path, 0, fin-debut+2);
	arg += debut;
	strncpy(path, arg, fin-debut+1);
	if (path[strlen(path)-1]=='-')
		path[strlen(path)-1]=0;
	if (path[strlen(path)-1]=='/' && strlen(path)>1)
		path[strlen(path)-1]=0;
	return(path);
}


