#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include "text.h"
#include "list.h"
#include "debug.h"
#include "user.h"
#include "conf.h"
#include "win.h"
#include "text.h"
#include "gtkftpd.h"



static GtkWidget *nb=NULL;
static GtkWidget *tbl=NULL;
static int nbuser=0;
GtkWidget *btn1=NULL, *btn2=NULL;

static gboolean create_tags(GtkTextBuffer *buffer);
static gboolean init_notebook();
static gboolean init_whichuser_text(GtkButton *btn, gpointer data);
static void desinit_whichuser_text(GtkButton *btn, gpointer data);
static gboolean verif_quota_line(Tuser *user);
static gboolean update_star_in_note(Tuser *user);
static gboolean switch_page(GtkNotebook *notebook, GtkNotebookPage *page, gint old_page, gpointer data);
static void scroll_to_end(Tuser *user);



gboolean init_text()
{
	if (!init_notebook())
		return(FALSE);
	tbl = gtk_table_new(2, 2, FALSE);
	btn1 = gtk_button_new_with_label("<-");
	btn2 = gtk_button_new_with_label("->");
	gtk_widget_set_usize(btn1, 40, 40);
	gtk_widget_set_usize(btn2, 40, 40);
	g_signal_connect(btn1, "clicked", G_CALLBACK(init_whichuser_text), NULL);
	g_signal_connect(btn2, "clicked", G_CALLBACK(desinit_whichuser_text), NULL);
	gtk_table_attach(GTK_TABLE(tbl), nb, 0, 1, 0, 2, GTK_FILL | GTK_EXPAND,
									GTK_FILL | GTK_EXPAND, 5, 0);
	gtk_table_attach(GTK_TABLE(tbl), btn1, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
	gtk_table_attach(GTK_TABLE(tbl), btn2, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
	return(TRUE);
}


static gboolean init_notebook()
{
	nb = gtk_notebook_new();
	gtk_notebook_popup_enable(GTK_NOTEBOOK(nb));
	gtk_notebook_set_scrollable(GTK_NOTEBOOK(nb), FALSE);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(nb), FALSE);
	g_signal_connect(nb, "switch-page", G_CALLBACK(switch_page), NULL);
	return(TRUE);
}


static gboolean init_whichuser_text(GtkButton *btn, gpointer data)
{
	Tuser *user;
	GtkWidget *db;

	user = get_selected_user();
	if (user)
	{
		if (!user->text->text)
		{
			if (!draw_user_text(user) )
				return(FALSE);
		} else {
			db = gtk_message_dialog_new(GTK_WINDOW(get_win()), 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
					"The user's note ever exists");
			g_signal_connect_swapped(GTK_OBJECT(db), "response", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(db));
			gtk_widget_show(db);
		}
	}
	return(TRUE);
}


static void desinit_whichuser_text(GtkButton *btn, gpointer data)
{
	int no, i;
	Tuser *user;
	GtkWidget *db;

	no = gtk_notebook_get_current_page(GTK_NOTEBOOK(nb));
	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user->text && user->text->nbpage==no)
		{
			if (user->id)
			{
				desinit_user_text(user);
				user->text->color = TRUE;
				nbuser--;
				return;
			} else {
				db = gtk_message_dialog_new(GTK_WINDOW(get_win()), 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
											"You can't delete this note !");
				g_signal_connect_swapped(GTK_OBJECT(db), "response", G_CALLBACK(gtk_widget_destroy), GTK_OBJECT(db));
				gtk_widget_show(db);
			}
		}
	}
}


gboolean init_user_text(Tuser *user)
{
	user->text = g_malloc(sizeof(Tuser_text));
	memset(user->text, 0, sizeof(Tuser_text));
	user->text->color = FALSE;
	user->text->text = NULL;
	user->text->buffer = NULL;
	user->text->nbpage = -1;
	user->text->sw = NULL;
	return(TRUE);
}


gboolean draw_user_text(Tuser *user)
{
	GtkWidget *lbl_onglet;
	
	user->text->text = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(user->text->text), GTK_WRAP_WORD);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(user->text->text), FALSE);
	gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(user->text->text), FALSE);
	user->text->buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(user->text->text));
	create_tags(user->text->buffer);
	user->text->sw = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW(user->text->sw), GTK_POLICY_AUTOMATIC,
									GTK_POLICY_AUTOMATIC);
	//gtk_scrolled_window_set_placement( GTK_SCROLLED_WINDOW(user->text->sw), GTK_CORNER_BOTTOM_LEFT);
	gtk_container_add(GTK_CONTAINER(user->text->sw), user->text->text);
	lbl_onglet = gtk_label_new(user->name);
	gtk_notebook_append_page(GTK_NOTEBOOK(nb), user->text->sw, lbl_onglet);
	gtk_widget_show_all(nb);
	user->text->nbpage = nbuser++;
	if (user->id==0)
		user_insert_sys_text(user, WELCOME_MSG);
	gtk_notebook_set_page(GTK_NOTEBOOK(nb), user->text->nbpage);
	return(TRUE);
}


void desinit_user_text(Tuser *user)
{
	gtk_notebook_remove_page(GTK_NOTEBOOK(nb), user->text->nbpage);
	free(user->text);
	init_user_text(user);
}


void desinit_text()
{
	if (btn1)
		gtk_widget_destroy(btn1);
	if (btn2)
		gtk_widget_destroy(btn2);
	if (nb)
		gtk_widget_destroy(nb);
	if (tbl)
		gtk_widget_destroy(tbl);
	nbuser=0;
}

GtkWidget *get_text()
{
	return(tbl);
}

static gboolean create_tags(GtkTextBuffer *buffer)
{
	gtk_text_buffer_create_tag(buffer, "vip", "foreground", "red", NULL);
	gtk_text_buffer_create_tag(buffer, "sys", "foreground", "darkgreen", NULL);
	gtk_text_buffer_create_tag(buffer, "name", "foreground", "blue", NULL);
	gtk_text_buffer_create_tag(buffer, "s_cmd", "style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_buffer_create_tag(buffer, "l_cmd", "weight", PANGO_WEIGHT_BOLD, NULL);
	return(TRUE);
}


gboolean user_insert_text(Tuser *user, char *txt, int tag)
{
	Tuser *user_a;
	GtkTextIter iter;
	char *name;
	char name_end[]=" -> ";
	char ip[18];
	int nbchar;

	user_a = (Tuser *) (g_slist_nth(list_user, 0))->data;
	gtk_text_buffer_get_end_iter(user_a->text->buffer, &iter);
	if (user->rezo && user->rezo->path && user->rezo->path->current)
	{
		name = (char *) malloc(strlen(user->name)+strlen(name_end)+18+1+strlen(user->rezo->path->current));
		memset(name, 0, strlen(user->name)+strlen(name_end)+18+1+strlen(user->rezo->path->current));
		strncpy(name, user->name, strlen(user->name));
		memset(ip, 0, 18);
		sprintf(ip, "(%d.%d.%d.%d)", user->rezo->ip[0], user->rezo->ip[1], user->rezo->ip[2], user->rezo->ip[3]);
		strncat(name, ip, strlen(ip));
		if (user->rezo->path && user->rezo->path->current)
			strncat(name, user->rezo->path->current, strlen(user->rezo->path->current));
	} else {
		name = (char *) malloc(strlen(user->name)+strlen(name_end)+18+1);
		memset(name, 0, strlen(user->name)+strlen(name_end)+18+1);
		strncpy(name, user->name, strlen(user->name));
	}
	strncat(name, name_end, strlen(name_end));

	if (!user->text)		
		return(FALSE);
	gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, name, -1, "name", NULL);
	free(name);
	if (!user->text->color)
	{
		if (tag==SMALL_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, txt, -1, "s_cmd", NULL);
		if (tag==LARGE_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, txt, -1, "l_cmd", NULL);
		if (!tag)
			gtk_text_buffer_insert(user_a->text->buffer, &iter, txt, -1);
	} else {
		if (tag==SMALL_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, txt, -1, "s_cmd", "vip", NULL);
		if (tag==LARGE_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, txt, -1, "l_cmd", "vip",  NULL);
		if (!tag)
			gtk_text_buffer_insert_with_tags_by_name(user_a->text->buffer, &iter, txt, -1, "vip", NULL);
	}
	gtk_text_buffer_insert(user_a->text->buffer, &iter, "\n", -1);

	if (user->text && user->text->text)
	{
		gtk_text_buffer_get_end_iter(user->text->buffer, &iter);
		nbchar = gtk_text_buffer_get_char_count(user->text->buffer);
		if (nbchar)
			gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, "\n", -1, "name", NULL);
		if (user->rezo && user->rezo->path && user->rezo->path->current)
			gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, user->rezo->path->current, -1, "name", NULL);
		gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, name_end, -1, "name", NULL);
		if (tag==SMALL_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, txt, -1, "s_cmd", NULL);
		if (tag==LARGE_TAG)
			gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, txt, -1, "l_cmd", NULL);
		if (!tag)
			gtk_text_buffer_insert(user->text->buffer, &iter, txt, -1);
	}
	if (!verif_quota_line(user))
		return(FALSE);
	update_star_in_note(user);
	scroll_to_end(user);
	scroll_to_end(user_a);
	return(TRUE);
}


static void scroll_to_end(Tuser *user)
{
	GtkTextIter iter;

	if (user && user->text && user->text->buffer)
	{
		gtk_text_buffer_get_end_iter(user->text->buffer, &iter);
		gtk_text_view_scroll_to_iter(GTK_TEXT_VIEW(user->text->text), &iter, 0, TRUE, 0.0, 1.0);
	}
}


gboolean user_insert_sys_text(Tuser *user, char *text)
{
	GtkTextIter iter;

	if (user->text->buffer)
	{
		gtk_text_buffer_get_end_iter(user->text->buffer, &iter);
		gtk_text_buffer_insert_with_tags_by_name(user->text->buffer, &iter, text, -1, "sys", NULL);
		gtk_text_buffer_insert(user->text->buffer, &iter, "\n", -1);
		if (!update_star_in_note(NULL))
			return(FALSE);
	}
	scroll_to_end(user);
	return(TRUE);
}


gboolean activate_color(Tuser *user)
{
	Tuser *user_other;
	int i;

	user->text->color = TRUE;
	for (i=0;i<g_slist_length(list_user);i++)
	{
		user_other = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user_other->text && user_other->id!=user->id)
			user_other->text->color = FALSE;
	}
	return(TRUE);
}


gboolean update_history_color(Tuser *user)
{
	Tuser *user_a;
	GtkTextIter beg_iter, end_iter;
	int nbline, i;
	char *txt;

	user_a = (Tuser *) (g_slist_nth(list_user, 0))->data;
	gtk_text_buffer_get_bounds(user_a->text->buffer, &beg_iter, &end_iter);
	gtk_text_buffer_remove_tag_by_name(user_a->text->buffer, "vip", &beg_iter, &end_iter);
	
	nbline = gtk_text_buffer_get_line_count(user_a->text->buffer);
	for (i=0;i<nbline;i++)
	{
		gtk_text_buffer_get_iter_at_line(user_a->text->buffer, &beg_iter, i);
		end_iter = beg_iter;
		if (!gtk_text_iter_forward_line(&end_iter))
			gtk_text_buffer_get_end_iter(user_a->text->buffer, &end_iter);
		txt = gtk_text_iter_get_text(&beg_iter, &end_iter);
		if (!strncmp(txt, user->name, strlen(user->name)) && *(txt+strlen(user->name))!='[' )
			gtk_text_buffer_apply_tag_by_name(user_a->text->buffer, "vip", &beg_iter, &end_iter);
	}
	return(TRUE);
}


static gboolean verif_quota_line(Tuser *user)
{
	Tuser *user_a;
	int nbline;
	GtkTextIter beg_iter, end_iter;

	user_a = (Tuser *) (g_slist_nth(list_user, 0))->data;
	if (!get_setting(HISTORY_BALISE))
	{
		fprintf(stderr, "Error: no history balise in config file\n");
		quitte_main_boucle();
		return(FALSE);
	}
	if (user_a->text->buffer)
	{
		nbline = gtk_text_buffer_get_line_count(user_a->text->buffer);
		if (nbline>atoi(get_setting(HISTORY_BALISE)))
		{
			gtk_text_buffer_get_start_iter(user_a->text->buffer, &beg_iter);
			end_iter = beg_iter;
			gtk_text_iter_forward_line(&end_iter);
			gtk_text_buffer_delete(user_a->text->buffer, &beg_iter, &end_iter);
		}
	}
	if (user->text && user->text->buffer)
	{
		nbline = gtk_text_buffer_get_line_count(user->text->buffer);
		if (nbline>atoi(get_setting(HISTORY_BALISE)))
		{
			gtk_text_buffer_get_start_iter(user->text->buffer, &beg_iter);
			end_iter = beg_iter;
			gtk_text_iter_forward_line(&end_iter);
			gtk_text_buffer_delete(user->text->buffer, &beg_iter, &end_iter);
		}
	}
	return(TRUE);
}



static gboolean update_star_in_note(Tuser *user)
{
	Tuser *user_a;
	GtkWidget *page;
	char *tab;

	if (!g_slist_length(list_user))		return(TRUE);
	user_a = (Tuser *) (g_slist_nth(list_user, 0))->data;
	if (user_a->text->buffer)
	{
		if (gtk_notebook_current_page(GTK_NOTEBOOK(nb))!=user_a->text->nbpage)
		{
			tab = (char *) malloc(strlen(user_a->name)+2);
			memset(tab, 0, strlen(user_a->name)+2);
			strncpy(tab, user_a->name, strlen(user_a->name));
			strncat(tab, "*", 1);
			page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), user_a->text->nbpage);
			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(nb), page, tab);
			free(tab);
		}
	}
	
	if (user && user->text->buffer)
	{
		if (gtk_notebook_current_page(GTK_NOTEBOOK(nb))!=user->text->nbpage)
		{
			tab = (char *) malloc(strlen(user->name)+2);
			memset(tab, 0, strlen(user->name)+2);
			strncpy(tab, user->name, strlen(user->name));
			strncat(tab, "*", 1);
			page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), user->text->nbpage);
			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(nb), page, tab);
			free(tab);
		}
	}

	return(TRUE);
}


static gboolean switch_page(GtkNotebook *notebook, GtkNotebookPage *page, gint page_index, gpointer data)
{
	int i;
	Tuser *user;
	GtkWidget *pag;

	for (i=0;i<g_slist_length(list_user);i++)
	{
		user = (Tuser *) (g_slist_nth(list_user, i))->data;
		if (user->text && user->text->buffer && user->text->nbpage==page_index)
		{
			pag = gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), page_index);
			gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(nb), pag, user->name);
		}
	}
	return(TRUE);
}
